/*************************************************************************
 GRANVIL software:  May, 2010
 
 Contributors:
 * Andrew P Morris amorris@well.ox.ac.uk
 * Reedik Magi reedik@well.ox.ac.uk
 
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *************************************************************************/
#include <iostream>
#include <algorithm>
#include <string>
#include <vector>
#include <fstream>
#include <cctype> // std::toupper
#include <map>
#include <zlib.h>
#include "sample.h"
#include "marker.h"
#include "global.h"
#include "structures.h"
#include "regression.h"
#include "studenttdistr.h"
#include "chisquaredistr.h"
#include "CmdLine.h"
#include "tools.h"

#define LENS 1000000
using namespace TCLAP;

double dabs(double x){if (x>0) return x; return x*-1.0;} //absolut value for doubles


using namespace std;
class gene{
public:
    std::string name;
    vector<marker> M;
    int chr;
    int start;
    int end;
};
//bool readGenotypeData(global & G, vector <sample> & S, vector <marker> & M, ofstream & LOG, ofstream & ERR);
//bool readGenotypeData(global & G, vector <sample> & S, vector <marker> & M, ofstream & LOG, ofstream & ERR, vector<gene> & GENE);
bool readGenotypeData(global & G, vector <sample> & S,  vector <gene> & GENE,  ofstream & LOG, ofstream & ERR, ofstream & OUT);
bool readGenotypeData(global & G, vector <sample> & S,  vector <gene> & GENE,  ofstream & LOG, ofstream & ERR, ofstream & OUTM, ofstream & OUTF);
void printModel( matrixD * X , arrayD * Y,ofstream & GENEPRINT);

bool readMapData(global & G, vector <gene> & GENE, ofstream & LOG, ofstream & ERR);
bool readSampleData(global & G, vector <sample> & S,  ofstream & LOG, ofstream & ERR);
bool readExclusionSampleData(global & G, vector <sample> & S,  ofstream & LOG, ofstream & ERR);
bool readExtractSampleData(global & G, vector <sample> & S,  ofstream & LOG, ofstream & ERR);
bool readExclusionMarkersData(global & G, vector <sample> & S,  ofstream & LOG, ofstream & ERR);
bool readExtractMarkersData(global & G, vector <sample> & S,  ofstream & LOG, ofstream & ERR);
bool resolveCommandLine(int argc, char *argv[], global & G);
void printVersion(string version);
void printHelp(string version);
void LowMemPrintGene(global & GLOBAL, vector <sample> & S, gene & GENE_p, ofstream & LOG, ofstream & ERR, ofstream & OUT);
void LowMemPrintGene(global & GLOBAL, vector <sample> & S, gene & GENE_p, ofstream & LOG, ofstream & ERR, ofstream & OUTM, ofstream & OUTF);
bool readConditionalData(global & GLOBAL, vector <sample> & S, ofstream & LOG,ofstream & ERR,gene & GENE_p);
bool readConditionalMarker(global & GLOBAL, vector <sample> & S,string marker,ofstream &  LOG,ofstream &  ERR, vector <double> & _marker);
bool readConditionalMarkerPos(global & GLOBAL, vector <sample> & S,string markerPos,ofstream &  LOG,ofstream &  ERR, vector <double> & _markerPos);


int Tokenize(const  string& str1, vector<string>& tokens, const string& delimiters = " ");

int main(int argc, char *argv[])
{
	global GLOBAL;
	vector <marker> M;
	vector <sample> S;
	vector <gene> GENE;
	//resolveCommandLine(argc, argv, GLOBAL);
    try {  
        CmdLine cmd("For more info: http://www.well.ox.ac.uk/GRANVIL/", ' ', GLOBAL.version );
    
        //
        // Define arguments
        //
        vector<string> methods_allowed;
        methods_allowed.push_back("threshold");
        methods_allowed.push_back("expected");
        ValuesConstraint<string> allowedVals( methods_allowed );
        
        ValueArg<string> s_ex_fArg("","exclude_samples","This specifies file with excluded samples",false,"","string", cmd);
        ValueArg<string> m_ex_fArg("","exclude_markers","This specifies file with excluded markers",false,"","string", cmd);
        ValueArg<string> s_in_fArg("","extract_samples","This specifies file with extracted samples",false,"","string", cmd);
        ValueArg<string> m_in_fArg("","extract_markers","This specifies file with extracted markers",false,"","string", cmd);
        
        ValueArg<string> chrArg("","chr","All markers in genotype file are forced to be from this chromosome. Ignoring the first column in genotype file",false,"","string", cmd);
        ValueArg<string> naArg("","missing_code","This specifies the coding for missing data (default NA)",false,"NA","double", cmd);
        ValueArg<double> imp_thresh_Arg("", "imp_thresh", "Inputation score threshold for including markers (default 0.4)", false, 0.4,"double" , cmd);
        ValueArg<double> call_thresh_Arg("", "call_thresh", "Call-rate threshold for the best guess genotypes (default 0.9)", false, 0.9,"double" , cmd); 
        ValueArg<double> rare_thresh_Arg("r", "rare_thresh", "Minor allele cutoff for defining rare variants (default 0.05)", false, 0.05,"double" , cmd); 
 
        MultiArg<string> covnamesArg("","cov_name", "Name of covariate to use (in case of several covariates, use this command multiple times i.e. --cov_name SEX --cov_name AGE etc.)", false, "string", cmd);
        
        MultiArg<string> covGeneArg("", "cond_gene", "Name of gene to use as a covariant in conditional analysis (in case of several genes, use this command multiple times)", false, "string" , cmd); 
        
        MultiArg<string> covMarkerArg("", "cond_marker", "Name of marker to use as a covariant in conditional analysis (in case of several markers, use this command multiple times)", false, "string" , cmd); 
        
        MultiArg<string> covMarkerPosArg("", "cond_marker_pos", "Positions of marker to be used as a covariant in conditional analysis (in case of several marker positions, use this command multiple times)", false, "string" , cmd); 
        
        SwitchArg covallArg("", "cov_all","All covariates are used in analysis", cmd);
        SwitchArg debugArg("", "debug","Debug mode enabled", cmd); 
        ValueArg<string> phenoArg("p","pheno","This specifies phenotype to test",true,"","string", cmd);
        
        ValueArg<string> sexArg("","sex","This specifies gender column name in sample file for sex stratified analysis (men=0, women=1)",false,"","string", cmd);
        MultiArg<string> printGeneArg("", "print_gene", "Name of gene model to be printed (in case of several genes, use this command multiple times)", false, "string" , cmd); 
        
        ValueArg<string> methodArg("m","method","This option controls how the genotype uncertainity is taken into account",true,"threshold",&allowedVals, cmd);
        ValueArg<string> outfArg("o","out","This specifies output files root",false,"granvil","string", cmd);
        ValueArg<string> mapfArg("x","genmap","This specifies gene map file",true,"","string", cmd);
        ValueArg<string> samplefArg("s","sample","This specifies sample file",true,"","string", cmd);
        ValueArg<string> genofArg("g","gen","This specifies genotype file",true,"","string", cmd);
        ValueArg<double> flankingArg("f","flanking","This specifies flanking region size in kb (default 0 kb)",false,0.0,"double", cmd);
        cmd.parse(argc,argv);
        
        GLOBAL.flanking = flankingArg.getValue();
        GLOBAL.inputSampleFile = samplefArg.getValue();
        GLOBAL.inputGenFile = genofArg.getValue();
        GLOBAL.inputMapFile = mapfArg.getValue();
        GLOBAL.inputExclusionMarkersFile = m_ex_fArg.getValue();
        GLOBAL.inputExclusionSamplesFile = s_ex_fArg.getValue();
        GLOBAL.inputExtractMarkersFile = m_in_fArg.getValue();
        GLOBAL.inputExtractSamplesFile = s_in_fArg.getValue();
        GLOBAL.outputRoot = outfArg.getValue();
        GLOBAL.outputError = outfArg.getValue() + ".err";
        GLOBAL.outputLog = outfArg.getValue() + ".log";
        GLOBAL.outputResult = outfArg.getValue() + ".result";
        GLOBAL.pheno = phenoArg.getValue();
        GLOBAL.cov_all = covallArg.getValue();
        GLOBAL.covList = covnamesArg.getValue();
        GLOBAL.covCount = (int)GLOBAL.covList.size();
        GLOBAL.debugMode = debugArg.getValue();
        if (uc(methodArg.getValue())=="THRESHOLD") GLOBAL.method=0;
        else {GLOBAL.method=1;GLOBAL.dosages=1;}
        GLOBAL.callrateThreshold = call_thresh_Arg.getValue();
        GLOBAL.rareVariantThreshold = rare_thresh_Arg.getValue();
        GLOBAL.infoThreshold = imp_thresh_Arg.getValue();
        if (naArg.getValue()!="") GLOBAL.missingCode = naArg.getValue();
        else GLOBAL.missingCode = "NA";

        if (sexArg.getValue()!="") GLOBAL.sexCode = sexArg.getValue();
        else GLOBAL.sexCode = "";
        GLOBAL.printGene = printGeneArg.getValue();

        GLOBAL.covGene = covGeneArg.getValue();
        GLOBAL.covMarker = covMarkerArg.getValue();
        GLOBAL.covMarkerPos = covMarkerPosArg.getValue();
        
        if (chrArg.getValue() == "") GLOBAL.chromosome=0;
        else
        {
            if (uc(chrArg.getValue())=="MT"){GLOBAL.chromosome=26;}
            else if (uc(chrArg.getValue())=="XY"){GLOBAL.chromosome=25;}
            else if (uc(chrArg.getValue())=="Y"){GLOBAL.chromosome=24;}
            else if (uc(chrArg.getValue())=="X"){GLOBAL.chromosome=23;}
            else if (atoi(chrArg.getValue().c_str())>0 && atoi(chrArg.getValue().c_str())< 27){GLOBAL.chromosome = atoi(chrArg.getValue().c_str());}
            else
            {
                cerr << "Unreadable chromosome number. Please use either numbers between 1-26 or X,Y,XY,MT"<<endl;
                exit(1);
                
            }
        }
        
    } catch (ArgException &e)  // catch any exceptions
	{ cerr << "error: " << e.error() << " for arg " << e.argId() << endl; }
    
    GLOBAL.createOutput();
	ofstream ERR (GLOBAL.outputError.c_str());
	ofstream LOG (GLOBAL.outputLog.c_str());
    
    
    
    string v; double len =17 -  GLOBAL.version.size()-1;
	int f = (int) ((len/2.0)+0.6); 	int r = (int) ((len/2.0));	for (int i=0; i < f;i++)v+=" ";
	v+='v';	v+=GLOBAL.version; for (int i=0; i < r;i++)v+=" ";
	LOG << endl;
	LOG << "@----------------------------------------------------------@" << endl;
	LOG << "|     GRANVIL        |" << v << "|  September, 2012  |" << endl;
	LOG << "|----------------------------------------------------------|" << endl;
	LOG << "|           (C) 2008 Reedik Magi & Andrew P Morris         |" << endl;
	LOG << "|               GNU General Public License, v2             |" << endl;
	LOG << "|----------------------------------------------------------|" << endl;
	LOG << "|   GRANVIL - Gene- or Region-based ANanlysis of Variants  |" << endl; 
	LOG << "|             of Intermediate and Low frequency            |" << endl;
	LOG << "|                                                          |" << endl;
	LOG << "|  For documentation, citation & bug-report instructions:  |" << endl;
	LOG << "|            http://www.well.ox.ac.uk/GRANVIL/             |" << endl;
	LOG << "@----------------------------------------------------------@" << endl;
	LOG << endl;
	LOG << "Sample file: " << GLOBAL.inputSampleFile << endl;
	LOG << "Genotype file: " << GLOBAL.inputGenFile << endl;
	LOG << "Gene map file: " << GLOBAL.inputMapFile << endl;
	if (GLOBAL.inputExclusionMarkersFile.compare("")!=0)LOG << "Exclusion markers file: " << GLOBAL.inputExclusionMarkersFile << endl;
	if (GLOBAL.inputExtractMarkersFile.compare("")!=0)LOG << "Inclusion markers file: " << GLOBAL.inputExtractMarkersFile << endl;
	if (GLOBAL.inputExclusionSamplesFile.compare("")!=0)LOG << "Exclusion sample file: " << GLOBAL.inputExclusionSamplesFile << endl;
	if (GLOBAL.inputExtractSamplesFile.compare("")!=0)LOG << "Inclusion samples file: " << GLOBAL.inputExtractSamplesFile << endl;
	
    LOG << "Phenotype name: " << GLOBAL.pheno << endl;
	if (GLOBAL.cov_all) LOG << "Using all covariates from phenotype file (cov_all)" << endl;
	else if (GLOBAL.covCount>-1) 
    {LOG << "Using covariates: ";
        for (int i = 0; i<GLOBAL.covCount; i++)
        {
        LOG << GLOBAL.covList[i] << " ";
        }
     LOG   << endl;
    }
    if (GLOBAL.covGene.size()>0){
        LOG << "Conditioning for the gene(s):";
        cout << "Conditioning for the gene(s):";
        for (int i=0; i<GLOBAL.covGene.size(); i++){LOG << " " << GLOBAL.covGene[i];cout << " " << GLOBAL.covGene[i]; }
        LOG << endl; cout << endl;
    }
    if (GLOBAL.covMarker.size()>0){
        LOG << "Conditioning for the marker(s):";
        cout << "Conditioning for the marker(s):";
        for (int i=0; i<GLOBAL.covMarker.size(); i++){LOG << " " << GLOBAL.covMarker[i];cout << " " << GLOBAL.covMarker[i]; }
        LOG << endl; cout << endl;
    }
    if (GLOBAL.covMarkerPos.size()>0){
        LOG << "Conditioning for the marker positions(s):";
        cout << "Conditioning for the marker position(s):";
        for (int i=0; i<GLOBAL.covMarkerPos.size(); i++){LOG << " " << GLOBAL.covMarkerPos[i];cout << " " << GLOBAL.covMarkerPos[i]; }
        LOG << endl; cout << endl;
    }
	if (GLOBAL.method==1) LOG << "Using allele dosages instead of determined genotypes (method=expected)" << endl;
    else{LOG<< "Using certain genotypes (method=threshold)"<<endl;}
    if (GLOBAL.debugMode) cout << "Method: " << GLOBAL.method << endl;
	if (GLOBAL.chromosome>0)LOG << "All markers from chromosome: "<<GLOBAL.chromosome << "(use this option, if first column does not contain chromosome number or you want to use imputed data as well)" << endl;
	LOG << "Marker allele freq. cut-off for determing rare variant <= " << GLOBAL.rareVariantThreshold << endl;
    if (GLOBAL.method==1) LOG << "Marker imputation info threshold >= " << GLOBAL.infoThreshold << endl;
    if (GLOBAL.method==0)LOG << "Marker callrate threshold >= " << GLOBAL.callrateThreshold << endl;
    LOG << "Missing phenotype data value: " << GLOBAL.missingCode << endl;
    LOG << "Using flanking region size: " << GLOBAL.flanking << " kb"<<endl;
	cout <<"Read sample data from " <<  GLOBAL.inputSampleFile << endl;
    if (!readSampleData(GLOBAL,S, LOG, ERR))
	{
		cout << "Error reading sample file" << endl;
		exit (0);
	}
    if (GLOBAL.quantitativeTrait)LOG << "Quantitative trait analysis (linear regression)" << endl;
	else LOG << "Case-control trait analysis (logistic regression)" << endl;
	
	if (!GLOBAL.inputExclusionSamplesFile.compare("")==0)		//exclusion list is used
	{
		if (!readExclusionSampleData(GLOBAL, S, LOG, ERR))
		{
			cout << "Error reading sample exclusion file" << endl;
			exit (0);
		}
	}
    
    if (!GLOBAL.inputExtractSamplesFile.compare("")==0)		//axtraction list is used
	{
		if (!readExtractSampleData(GLOBAL, S, LOG, ERR))
		{
			cout << "Error reading sample extraction file" << endl;
			exit (0);
		}
	}
    if (!GLOBAL.inputExclusionMarkersFile.compare("")==0)		//exclusion list is used
	{
		if (!readExclusionMarkersData(GLOBAL, S, LOG, ERR))
		{
			cout << "Error reading markers exclusion file" << endl;
			exit (0);
		}
        GLOBAL.markerExclusion=true;
	}
    
    if (!GLOBAL.inputExtractMarkersFile.compare("")==0)		//extraction list is used
	{
		if (!readExtractMarkersData(GLOBAL, S, LOG, ERR))
		{
			cout << "Error reading markers extraction file" << endl;
			exit (0);
		}
        GLOBAL.markerExtraction=true;
	}
    

 	cout <<"Read genemap data from " <<  GLOBAL.inputMapFile << endl;
    if (!readMapData(GLOBAL,GENE, LOG, ERR))
	{
		cout << "Error reading genemap file" << endl;
		exit (0);
	}   
    
    //Adding gene to conditional analysis
    for (int _curCov = 0; _curCov < GLOBAL.covGene.size(); _curCov++)
    {
        cout << "Reading gene " << GLOBAL.covGene[_curCov] << " information for conditional analysis"<<endl;
        vector<double> covData;
        vector <double> genotype, weight;
        int count = 0;
        int covLoc = -1;
        for (unsigned int i = 0; i < GENE.size(); i++)
        {
            if (GENE[i].name == GLOBAL.covGene[_curCov]){covLoc = i;}
        }
        if (covLoc==-1)
        {
            cout << "Gene name specified as covariate cannot be found from the gene list. Exit program!";
            exit(1);
        }
        cout <<"Read genotype data from " <<  GLOBAL.inputGenFile << endl;
        if (!readConditionalData(GLOBAL, S, LOG, ERR,GENE[covLoc]))
        {
            cout << "Error reading genotype file for conditional gene information" << endl;
            exit (1);
        }
        if (GLOBAL.debugMode)cout << "Conditional sample size: " << S.size() << " marker size: " << GENE[covLoc].M.size() << endl;
        for (unsigned int k = 0; k < S.size(); k++)
        {
            genotype.push_back(0);
            weight.push_back(0);
        }
        for (unsigned int j = 0; j < GENE[covLoc].M.size(); j++)
        {
            for (unsigned int k = 0; k < S.size(); k++)
            {
                if (!GLOBAL.dosages)
                {
                    if (GENE[covLoc].M[j].getGenotype(k)>0)
                    {
                        if(!GLOBAL.dosages)genotype[k]++;  //add one
                    }
                    else if(GENE[covLoc].M[j].getGenotype(k)!=0)
                    {
                        weight[k]++; // we have missing phenotype
                    }
                }
                else
                {
                    if (GENE[covLoc].M[j].getGenotype(k)>=0)
                    {
                        genotype[k]+= GENE[covLoc].M[j].getGenotype(k); //add dosage
                    }
                    else
                    {
                        weight[k]++;
                    }
                    
                }
            }
        count++;
        }
        int l = 0;
        for (unsigned int k = 0; k < S.size(); k++)
        {
   //         GLOBAL.conditionalData.push_back(genotype[k]/ (double)(count-weight[k]));
            if (S[k].isOK())
            {
                if (GLOBAL.debugMode)cout << "Adding covariate into vector for sample: " << S[k].getName() << endl;
                if (GLOBAL.debugMode)cout << "covariate count: " << S[k].getCovariateCount() << endl;                
                S[k].addCovariate(genotype[l]/ (double)(count-weight[l]));
                if (GLOBAL.debugMode)cout << "covariate count: " << S[k].getCovariateCount() << endl;
                l++;
            }
            else
            {
                S[k].addCovariate(-9999);
            }
        }
        GENE[covLoc].M.clear();
    }
    //Adding marker to conditional analysis
    for (int _curMarker = 0; _curMarker < GLOBAL.covMarker.size(); _curMarker++)
    {
        cout << "Reading marker " << GLOBAL.covMarker[_curMarker] << " information for conditional analysis"<<endl;
        vector <double> genotype;
        if (!readConditionalMarker(GLOBAL,S, GLOBAL.covMarker[_curMarker], LOG, ERR, genotype))
        {
            cout << "Error reading genotype file for conditional marker information" << endl;
            exit (1);
        }
        int l = 0;
        for (unsigned int k = 0; k < S.size(); k++)
        {
            //         GLOBAL.conditionalData.push_back(genotype[k]/ (double)(count-weight[k]));
            if (S[k].isOK())
            {
                S[k].addCovariate(genotype[l]);
                l++;
            }
            else
            {
                S[k].addCovariate(-9999);
            }
        }
        
    }
    //Adding marker positions to conditional analysis
    for (int _curMarkerPos = 0; _curMarkerPos < GLOBAL.covMarkerPos.size(); _curMarkerPos++)
    {
        cout << "Reading marker position " << GLOBAL.covMarkerPos[_curMarkerPos] << " information for conditional analysis"<<endl;
        vector <double> genotype;
        if (!readConditionalMarkerPos(GLOBAL,S, GLOBAL.covMarkerPos[_curMarkerPos], LOG, ERR, genotype))
        {
            cout << "Error reading genotype file for conditional marker position information" << endl;
            exit (1);
        }
        int l = 0;
        for (unsigned int k = 0; k < S.size(); k++)
        {
            //         GLOBAL.conditionalData.push_back(genotype[k]/ (double)(count-weight[k]));
            if (S[k].isOK())
            {
                S[k].addCovariate(genotype[l]);
                l++;
            }
            else
            {
                S[k].addCovariate(-9999);
            }
        }
        
    }

    if (GLOBAL.sexCode==""){
        ofstream OUT (GLOBAL.outputResult.c_str());

        OUT << "gene\tmarker_count\tsample_count\trare_variant_sum\ttotal_maf\taverage_maf\tbeta\tse\tz\tp"<<endl;
        readGenotypeData(GLOBAL, S,  GENE,  LOG, ERR, OUT);
    }
    else
    {
        ofstream OUTM (GLOBAL.outputResultM.c_str());
        ofstream OUTF (GLOBAL.outputResultF.c_str());
        OUTM << "gene\tmarker_count\tsample_count\trare_variant_sum\ttotal_maf\taverage_maf\tbeta\tse\tz\tp"<<endl;
        OUTF << "gene\tmarker_count\tsample_count\trare_variant_sum\ttotal_maf\taverage_maf\tbeta\tse\tz\tp"<<endl;
        readGenotypeData(GLOBAL, S,  GENE,  LOG, ERR, OUTM, OUTF);
    }
    cout << "We have " << GENE.size() << " gene regions!" << endl;
    LOG << "Gene regions read from file: " << GENE.size() << endl;


    cout << "Analysis finished."<<endl;
    ERR << "Analysis finished."<<endl;
    LOG << "Analysis finished."<<endl;

    
	ERR.close();
	LOG.close();
    return 0;
}


bool 
readSampleData(global & G, vector <sample> & S,  ofstream & LOG, ofstream & ERR)
{
    char sb [10];				//error name char buffer
	int lineNr = 0;
	ifstream F (G.inputSampleFile.c_str());
	if (F.is_open())
    {
		vector <string> columnNames;
		int phenoColumn;
		vector <int> covColumn;
		int okCnt = 0;
        int okMenCnt = 0;
        int okWomenCnt = 0;
       	while (! F.eof() )
        {
	        string line;
        	vector<string> tokens;
        	getline (F,line);
			string currentmarker = "";
			int n = Tokenize(string(line), tokens, " ");		//tabulating file by space
			if (lineNr>1)
			{
				if (n>2 && n>=phenoColumn && n>=G.sexColumn)
				{
					string sampleName = string(tokens[0]);
					double phenotype;
                    if (tokens[phenoColumn].c_str() != G.missingCode)
                    {
                        phenotype = atof(tokens[phenoColumn].c_str());
                    }
                    else
                    {
                        phenotype=-9999;
                    }
                    
					vector <double> covariates;
					sample s(sampleName);
					if (!G.quantitativeTrait)
					{
						if (phenotype == 0) s.setStatus(0);		//control
						else if (phenotype == 1) s.setStatus(1);	//case 
						else s.setStatus(-9999);
					}
					else if (phenotype == -9999)s.setPhenotype(-9999);
					else s.setPhenotype(phenotype);
                    if (G.sexColumn>=0)
                    {
                        if( atoi(tokens[G.sexColumn].c_str()) ==0 || atoi(tokens[G.sexColumn].c_str())==1 )
                        {
                            s.setSex(atoi(tokens[G.sexColumn].c_str()));
                        }
                        else
                        {
                            s.setSex(-9999);
                        }
                    }
					for (unsigned int i = 0; i < covColumn.size(); i++)
					{
						if (tokens[covColumn[i]].c_str()!=G.missingCode) 
                        {
                            s.addCovariate(atof(tokens[covColumn[i]].c_str()));
                        }
						else s.addCovariate(-9999);
					}
                    
					S.push_back(s);
					if (s.isOK())
					{
                        if (G.sexCode!="" && s.getSex()==-9999){}
                        else if (G.sexCode!="")
                        {
                            if (s.getSex()==0)okMenCnt++;
                            if (s.getSex()==1)okWomenCnt++;                            
                        }
                        okCnt ++;
					}
					
				}
			}
			else if (lineNr == 0)
			{
				if (n>2)
				{
					for (int i = 0; i < n; i++)
					{
						for (int j = 0; j<G.covCount; j++)
                        {
                            if (uc(tokens[i]) == uc(G.covList[j]) && uc(tokens[i]) != uc(G.sexCode))
                            {
                                covColumn.push_back(i); //cov_all option accoring to next row, not in here!!
                            }
                        }
						if (uc(tokens[i]) == uc(G.pheno))
						{
                                phenoColumn = i;
						}
                        if (uc(tokens[i]) == uc(G.sexCode))
						{
                            G.sexColumn = i;
						}
                        columnNames.push_back(tokens[i]);
					}
				}
                
			}
			else if (lineNr == 1)
			{
				if (n>2)
				{
					for (int i = 0; i < n; i++)
					{
                            if ((tokens[i][0] == 'C' || tokens[i][0] == 'D') && G.cov_all==true && G.sexColumn != i)
                            {
                                covColumn.push_back(i); //cov_all option accoring to this row!!
                            }
					}
                    if (tokens[phenoColumn]=="P"){G.quantitativeTrait=true;}
                    else if (tokens[phenoColumn]=="B"){G.quantitativeTrait=false;}
                    else 
                    {
                        cout << "Phenotype of interest has code " << tokens[phenoColumn] << " (must be either B or P)" << endl;
                        exit (1);
                    }
                    if (G.sexCode!="" && G.sexColumn==-1)
                    {
                        cout << "Cannot find gender with code: " << G.sexCode << " from sample file. Exit program."<< endl;
                        exit(1);
                    }
				}
                
                
			}

			lineNr++;
		}
        
		//
		//print some stats about phenotypes
		if (G.quantitativeTrait) cout << "Quantitative trait" << endl;
		else cout << "case-control trait" << endl;
		cout << "Number of samples with phenotype and covariate data (N): " << okCnt << endl;
		if (!G.quantitativeTrait)
		{
			cout << "Phenotype: " << columnNames[phenoColumn] << endl;
			int cases = 0;
			int controls = 0;
			for (unsigned int i = 0; i < S.size(); i ++)
			{
				if (S[i].isOK())
				{
					if (S[i].getStatus()==0)controls++;
					if (S[i].getStatus()==1)cases++;
				}
			}
			cout << "	Cases controls: " << cases << " "   << controls << endl;
		}
		else
		{
			cout << "Phenotype: " << columnNames[phenoColumn] << endl;
			double min = 1e100;
			double max = -1e100;
			double sum = 0;
			for (unsigned int i = 0; i < S.size(); i ++)
			{
				if (S[i].isOK())
				{
					if (S[i].getPhenotype() < min) min = S[i].getPhenotype();
					if (S[i].getPhenotype() > max) max = S[i].getPhenotype();
					sum += S[i].getPhenotype();
				}
			}
			cout << "	Min Aver Max: " << min << " "   << sum/okCnt << " "<< max<< endl;
		}
		if (covColumn.size()==0) cout << "No covariates"<<endl;
		else
		{
			for (unsigned int i = 0; i < covColumn.size(); i++)
			{
                cout << "Covariate: " << columnNames[covColumn[i]] << endl;
                double min = 1e100;
                double max = -1e100;
                double sum = 0;
                for (unsigned int j = 0; j < S.size(); j ++)
                {
                    if (S[j].isOK())
                    {
                        if (S[j].getCovariate(i) < min) min = S[j].getCovariate(i);
                        if (S[j].getCovariate(i) > max) max = S[j].getCovariate(i);
                        sum += S[j].getCovariate(i);
                    }
                }
                cout << "	Min Aver Max: " << min << " "   << sum/okCnt << " "<< max<< endl;
			}
		}
		G.pheno_ok = okCnt;
        G.pheno_men_ok = okMenCnt;
        G.pheno_women_ok = okWomenCnt;
        if (G.debugMode)cout << "Pheno_ok" << okCnt << " men " <<  okMenCnt << " women " << okWomenCnt << endl;
	}
	else 
	{
		cerr << "Error: Cannot find or access " << G.inputSampleFile << ". Exit program." << endl;
		sprintf(sb, "E%.8d" , G.errNr); G.errNr++;
		LOG << "Error: Cannot find or access " << G.inputSampleFile << ". Exit program! (" << sb << ")" <<endl;
		ERR << sb << " Error: Cannot find or access " << G.inputSampleFile << ". Program exit!" <<endl;
		ERR << sb << " Error: Make sure that this file is in given path and is readable." <<endl;
		return false;
	}
	
	return true;
}

bool 
readExclusionSampleData(global & G, vector <sample> & S,  ofstream & LOG, ofstream & ERR)
{
	int excludedCount = 0;
	char sb [10];				//error name char buffer
	ifstream F (G.inputExclusionSamplesFile.c_str());
	if (F.is_open())
    {
       	while (! F.eof() )
        {
	        string line;
        	vector<string> tokens;
        	getline (F,line);
			string currentmarker = "";
			int n = Tokenize(string(line), tokens, " ");		//tabulating file by space
			if (n>0)
			{
				string sampleName = string(tokens[0]);
				for (int i = 0; i < S.size(); i++)
				{
					if (sampleName.compare(S[i].getName())==0)
					{
						S[i].setPhenotype(-9999);
						S[i].setStatus(-9999);
						excludedCount++;
					}
				}
                
			}	
		}
	}
	else
	{
		cerr << "Error: Cannot find or access " << G.inputExclusionSamplesFile << ". Exit program." << endl;
		sprintf(sb, "E%.8d" , G.errNr); G.errNr++;
		LOG << "Error: Cannot find or access " << G.inputExclusionSamplesFile << ". Exit program! (" << sb << ")" <<endl;
		ERR << sb << " Error: Cannot find or access " << G.inputExclusionSamplesFile << ". Program exit!" <<endl;
		ERR << sb << " Error: Make sure that this file is in given path and is readable." <<endl;
		return false;
	}
	cout << "Excluded individuals count: " << excludedCount << endl;
	int okCnt =0;
    int okMenCnt = 0;
    int okWomenCnt = 0;
	for (int i =0; i<S.size(); i++){ 
        if(S[i].isOK()){
            if (G.sexCode!="" && S[i].getSex()==-9999){}
            else if (G.sexCode!="")
            {
                if (S[i].getSex()==0)okMenCnt++;
                if (S[i].getSex()==1)okWomenCnt++;                            
            }
            okCnt ++;
        }
    }
    
	cout << "Remining individuals: " << okCnt << endl;
	G.pheno_ok = okCnt;
    G.pheno_men_ok = okMenCnt;
    G.pheno_women_ok = okWomenCnt;
    
	return true;
}

bool 
readExtractSampleData(global & G, vector <sample> & S,  ofstream & LOG, ofstream & ERR)
{
	int excludedCount = 0;
	char sb [10];				//error name char buffer
    vector<string> sampleNames;
	ifstream F (G.inputExtractSamplesFile.c_str());
	if (F.is_open())
    {
       	while (! F.eof() )
        {
	        string line;
        	vector<string> tokens;
        	getline (F,line);
			string currentmarker = "";
			int n = Tokenize(string(line), tokens, " ");		//tabulating file by space
			if (n>0)
			{
				sampleNames.push_back(string(tokens[0]));
			}	
		}
        for (int i = 0; i < S.size(); i++)
        {
            bool sample_is_ok= false;
            for (int j = 0; j<sampleNames.size(); j++)
            {
                if (sampleNames[j].compare(S[i].getName())==0) sample_is_ok=true;
            
            }
            if (!sample_is_ok)
            {
                S[i].setPhenotype(-9999);
                S[i].setStatus(-9999);
                excludedCount++;
            }
        }
        
	}
	else
	{
		cerr << "Error: Cannot find or access " << G.inputExtractSamplesFile << ". Exit program." << endl;
		sprintf(sb, "E%.8d" , G.errNr); G.errNr++;
		LOG << "Error: Cannot find or access " << G.inputExtractSamplesFile << ". Exit program! (" << sb << ")" <<endl;
		ERR << sb << " Error: Cannot find or access " << G.inputExtractSamplesFile << ". Program exit!" <<endl;
		ERR << sb << " Error: Make sure that this file is in given path and is readable." <<endl;
		return false;
	}
	int okCnt =0;
    int okMenCnt = 0;
    int okWomenCnt = 0;
	for (int i =0; i<S.size(); i++){ 
        if(S[i].isOK()){
            if (G.sexCode!="" && S[i].getSex()==-9999){}
            else if (G.sexCode!="")
            {
                if (S[i].getSex()==0)okMenCnt++;
                if (S[i].getSex()==1)okWomenCnt++;                            
            }
            okCnt ++;
        }
    }
    
	cout << "Remining individuals: " << okCnt << endl;
	G.pheno_ok = okCnt;
    G.pheno_men_ok = okMenCnt;
    G.pheno_women_ok = okWomenCnt;
	return true;
}
bool 
readConditionalMarker(global & G, vector <sample> & S,string marker,ofstream &  LOG,ofstream &  ERR, vector <double> &  _marker)
{
    char sb [10];				//error name char buffer
    int forcedChromosome = G.chromosome; // I am forcing all markers from the input file to be be from predefined chromosome - snptest has "---" in first column for imputed snps instead of chromosome number
 
    if (G.inputGenFile.substr(G.inputGenFile.length()-2)=="gz")
    {
        gzFile F1 =gzopen(G.inputGenFile.c_str(),"r");
        char *buffer = new char[LENS];
        while(0!=gzgets(F1,buffer,LENS))
        {
                string line;
                vector<string> tokens;

                
                
                string currentmarker = "";
                int n = Tokenize(buffer, tokens, " ");		//tabulating file by space
                if (n>2)
                {
                    int chr;
                    if (uc(tokens[0])=="MT") chr=26;
                    else if (uc(tokens[0])=="XY") chr=25;
                    else if (uc(tokens[0])=="Y") chr=24;
                    else if (uc(tokens[0])=="X") chr=23;
                    else chr = atoi(tokens[0].c_str());
                    
                    if (G.debugMode) cout << "CONDITIONAL MARKER: Chromosome id: " << chr;
                    
                    if (forcedChromosome) 
                    {
                        chr = forcedChromosome;
                        if (G.debugMode) cout << "CONDITIONAL MARKER: Chromosome forced: " << chr;                    
                    }
                    int pos = atoi(tokens[2].c_str());
                    string markerName = string(tokens[1]);
                    char effectAllele = tokens[3][0];
                    char nonEffectAllele = tokens[4][0];
                    if (G.debugMode) cout << "CONDITIONAL MARKER: Pos: " << pos << "\nmarker:" << markerName << "\nea/nea:" << effectAllele <<"/" << nonEffectAllele<<"\n";
                    
                    vector <double> gen;
                    vector <double> gen2;
                    //                double fijeij = 0;                 //for info score
                    double aa=0; double aA=0; double AA=0;
                    double callrate=0; double ok_gen=0; double not_ok_gen=0;
                    double infoscore = 1;
                    int j = 0;
                    for (int i = 5; i < n; i+=3)
                    {
                        //determined
                        if (S[j].isOK())
                        {
                            if(G.dosages==false)
                            {
                                if (atof(tokens[i].c_str())>=.95){aa++;}
                                else if (atof(tokens[i+1].c_str())>=.95){aA++;}
                                else if (atof(tokens[i+2].c_str())>=.95){AA++;}
                                
                                if (atof(tokens[i].c_str())>=.95){gen.push_back(2.0);ok_gen++;}
                                else if (atof(tokens[i+1].c_str())>=.95){gen.push_back(1.0);ok_gen++;}
                                else if (atof(tokens[i+2].c_str())>=.95){gen.push_back(0.0);ok_gen++;}
                                else {gen.push_back(-9999);not_ok_gen++;}
                            }
                            else
                            {
                                aa+=atof(tokens[i].c_str());
                                aA+=atof(tokens[i+1].c_str());
                                AA+=atof(tokens[i+2].c_str());
                                gen2.push_back(((atof(tokens[i+2].c_str())) + (atof(tokens[i+1].c_str())/2)));
                                gen.push_back(((atof(tokens[i].c_str())) + (atof(tokens[i+1].c_str())/2)));
                            }
                        }
                        if (n < i+5) i+=3;		//avoiding space in line end
                        
                        j++;
                    }
                    if (G.debugMode)cout<<"CONDITIONAL: aa,aA,AA"<< aa<<","<<aA<<","<<AA<<"\n";
                    if (G.dosages)
                    {
                        callrate=1;
                        double eaf = 0;
                        if (aa+aA+AA>0)
                        {
                            eaf = ((2*aa)+aA)/(2*(aa+aA+AA));
                            
                        }
                        if (eaf==1 || eaf==0){infoscore=1;}
                        //info score calculation according to the measure in snptest:             
                        //https://mathgen.stats.ox.ac.uk/genetics_software/snptest/snptest.v2.pdf
                        
                    }
                    else 
                    {
                        callrate = ok_gen/(ok_gen+not_ok_gen);   
                        if (G.debugMode)cout<<"CONDITIONAL: Callrate: "<< callrate <<endl;
                    }
                    
                    std::stringstream mrk2;
                    mrk2 << chr << ":" << pos; 
                std:string mrkname = mrk2.str();
                    
                    if (aa+aA+AA>0 && markerName == marker)
                    {
                        _marker = gen;
                        return true;
                    }
                }
            }
    
    }
    else
    {
        ifstream F1 (G.inputGenFile.c_str());
        if (F1.is_open())
        {
            while (! F1.eof() )
            {
                string line;
                vector<string> tokens;
                getline (F1,line);
                
                
                string currentmarker = "";
                int n = Tokenize(string(line), tokens, " ");		//tabulating file by space
                if (n>2)
                {
                    int chr;
                    if (uc(tokens[0])=="MT") chr=26;
                    else if (uc(tokens[0])=="XY") chr=25;
                    else if (uc(tokens[0])=="Y") chr=24;
                    else if (uc(tokens[0])=="X") chr=23;
                    else chr = atoi(tokens[0].c_str());
                    
                    if (G.debugMode) cout << "CONDITIONAL MARKER: Chromosome id: " << chr;
                    
                    if (forcedChromosome) 
                    {
                        chr = forcedChromosome;
                        if (G.debugMode) cout << "CONDITIONAL MARKER: Chromosome forced: " << chr;                    
                    }
                    int pos = atoi(tokens[2].c_str());
                    string markerName = string(tokens[1]);
                    char effectAllele = tokens[3][0];
                    char nonEffectAllele = tokens[4][0];
                    if (G.debugMode) cout << "CONDITIONAL MARKER: Pos: " << pos << "\nmarker:" << markerName << "\nea/nea:" << effectAllele <<"/" << nonEffectAllele<<"\n";
                    
                    vector <double> gen;
                    vector <double> gen2;
    //                double fijeij = 0;                 //for info score
                    double aa=0; double aA=0; double AA=0;
                    double callrate=0; double ok_gen=0; double not_ok_gen=0;
                    double infoscore = 1;
                    int j = 0;
                    for (int i = 5; i < n; i+=3)
                    {
                        //determined
                        if (S[j].isOK())
                        {
                        if(G.dosages==false)
                        {
                            if (atof(tokens[i].c_str())>=.95){aa++;}
                            else if (atof(tokens[i+1].c_str())>=.95){aA++;}
                            else if (atof(tokens[i+2].c_str())>=.95){AA++;}
                            
                            if (atof(tokens[i].c_str())>=.95){gen.push_back(2.0);ok_gen++;}
                            else if (atof(tokens[i+1].c_str())>=.95){gen.push_back(1.0);ok_gen++;}
                            else if (atof(tokens[i+2].c_str())>=.95){gen.push_back(0.0);ok_gen++;}
                            else {gen.push_back(-9999);not_ok_gen++;}
                        }
                        else
                        {
                            aa+=atof(tokens[i].c_str());
                            aA+=atof(tokens[i+1].c_str());
                            AA+=atof(tokens[i+2].c_str());
                            gen2.push_back(((atof(tokens[i+2].c_str())) + (atof(tokens[i+1].c_str())/2)));
                            gen.push_back(((atof(tokens[i].c_str())) + (atof(tokens[i+1].c_str())/2)));
                        }
                        }
                        if (n < i+5) i+=3;		//avoiding space in line end
                        
                        j++;
                    }
                    if (G.debugMode)cout<<"CONDITIONAL: aa,aA,AA"<< aa<<","<<aA<<","<<AA<<"\n";
                    if (G.dosages)
                    {
                        callrate=1;
                        double eaf = 0;
                        if (aa+aA+AA>0)
                        {
                            eaf = ((2*aa)+aA)/(2*(aa+aA+AA));
                            
                        }
                        if (eaf==1 || eaf==0){infoscore=1;}
                        //info score calculation according to the measure in snptest:             
                        //https://mathgen.stats.ox.ac.uk/genetics_software/snptest/snptest.v2.pdf
                        
                    }
                    else 
                    {
                        callrate = ok_gen/(ok_gen+not_ok_gen);   
                        if (G.debugMode)cout<<"CONDITIONAL: Callrate: "<< callrate <<endl;
                    }
                    
                    std::stringstream mrk2;
                    mrk2 << chr << ":" << pos; 
                    string mrkname = mrk2.str();
                    
                    if (aa+aA+AA>0 && markerName == marker)
                    {
                        _marker = gen;
                        return true;
                    }
                }
            }
        }
        else 
        {
            cerr << "Error: Cannot find or access " << G.inputGenFile << ". Exit program." << endl;
            sprintf(sb, "E%.8d" , G.errNr); G.errNr++;
            LOG << "Error: Cannot find or access " << G.inputGenFile << ". Exit program! (" << sb << ")" <<endl;
            ERR << sb << " Error: Cannot find or access " << G.inputGenFile << ". Program exit!" <<endl;
            ERR << sb << " Error: Make sure that this file is in given path and is readable." <<endl;
            return false;
        }
        F1.close();
    }
    cerr << "Error: Cannot find conditional marker " << marker << ". Exit program." << endl;
    sprintf(sb, "E%.8d" , G.errNr); G.errNr++;
    LOG << "Error: Cannot find conditional marker " << marker << ". Exit program! (" << sb << ")" <<endl;
    ERR << sb << " Error: Cannot find conditional marker " << marker << ". Program exit!" <<endl;
    exit (1);
    return false;
}

bool 
readConditionalMarkerPos(global & G, vector <sample> & S,string marker,ofstream &  LOG,ofstream &  ERR, vector <double> &  _markerPos)
{
    char sb [10];				//error name char buffer
    int forcedChromosome = G.chromosome; // I am forcing all markers from the input file to be be from predefined chromosome - snptest has "---" in first column for imputed snps instead of chromosome number
    
    if (G.inputGenFile.substr(G.inputGenFile.length()-2)=="gz")
    {
        gzFile F1 =gzopen(G.inputGenFile.c_str(),"r");
        char *buffer = new char[LENS];
        while(0!=gzgets(F1,buffer,LENS))
        {
            string line;
            vector<string> tokens;
            
            
            
            string currentmarker = "";
            int n = Tokenize(buffer, tokens, " ");		//tabulating file by space
            if (n>2)
            {
                int chr;
                if (uc(tokens[0])=="MT") chr=26;
                else if (uc(tokens[0])=="XY") chr=25;
                else if (uc(tokens[0])=="Y") chr=24;
                else if (uc(tokens[0])=="X") chr=23;
                else chr = atoi(tokens[0].c_str());
                
                if (G.debugMode) cout << "CONDITIONAL MARKER POS: Chromosome id: " << chr;
                
                if (forcedChromosome) 
                {
                    chr = forcedChromosome;
                    if (G.debugMode) cout << "CONDITIONAL MARKER POS: Chromosome forced: " << chr;                    
                }
                string markerPos = string(tokens[2]);
                string markerName = string(tokens[1]);
                char effectAllele = tokens[3][0];
                char nonEffectAllele = tokens[4][0];
                if (G.debugMode) cout << "CONDITIONAL MARKER POS: Pos: " << markerPos << "\nmarker:" << markerName << "\nea/nea:" << effectAllele <<"/" << nonEffectAllele<<"\n";
                
                vector <double> gen;
                vector <double> gen2;
                //                double fijeij = 0;                 //for info score
                double aa=0; double aA=0; double AA=0;
                double callrate=0; double ok_gen=0; double not_ok_gen=0;
                double infoscore = 1;
                int j = 0;
                for (int i = 5; i < n; i+=3)
                {
                    //determined
                    if (S[j].isOK())
                    {
                        if(G.dosages==false)
                        {
                            if (atof(tokens[i].c_str())>=.95){aa++;}
                            else if (atof(tokens[i+1].c_str())>=.95){aA++;}
                            else if (atof(tokens[i+2].c_str())>=.95){AA++;}
                            
                            if (atof(tokens[i].c_str())>=.95){gen.push_back(2.0);ok_gen++;}
                            else if (atof(tokens[i+1].c_str())>=.95){gen.push_back(1.0);ok_gen++;}
                            else if (atof(tokens[i+2].c_str())>=.95){gen.push_back(0.0);ok_gen++;}
                            else {gen.push_back(-9999);not_ok_gen++;}
                        }
                        else
                        {
                            aa+=atof(tokens[i].c_str());
                            aA+=atof(tokens[i+1].c_str());
                            AA+=atof(tokens[i+2].c_str());
                            gen2.push_back(((atof(tokens[i+2].c_str())) + (atof(tokens[i+1].c_str())/2)));
                            gen.push_back(((atof(tokens[i].c_str())) + (atof(tokens[i+1].c_str())/2)));
                        }
                    }
                    if (n < i+5) i+=3;		//avoiding space in line end
                    
                    j++;
                }
                if (G.debugMode)cout<<"CONDITIONAL MARKER POS: aa,aA,AA"<< aa<<","<<aA<<","<<AA<<"\n";
                if (G.dosages)
                {
                    callrate=1;
                    double eaf = 0;
                    if (aa+aA+AA>0)
                    {
                        eaf = ((2*aa)+aA)/(2*(aa+aA+AA));
                        
                    }
                    if (eaf==1 || eaf==0){infoscore=1;}
                    //info score calculation according to the measure in snptest:             
                    //https://mathgen.stats.ox.ac.uk/genetics_software/snptest/snptest.v2.pdf
                    
                }
                else 
                {
                    callrate = ok_gen/(ok_gen+not_ok_gen);   
                    if (G.debugMode)cout<<"CONDITIONAL MARKER POS: Callrate: "<< callrate <<endl;
                }
                
                std::stringstream mrk2;
                mrk2 << chr << ":" << markerPos; 
            std:string mrkname = mrk2.str();
                
                if (aa+aA+AA>0 && markerPos == marker)
                {
                    _markerPos = gen;
                    return true;
                }
            }
        }
        
    }
    else
    {
        ifstream F1 (G.inputGenFile.c_str());
        if (F1.is_open())
        {
            while (! F1.eof() )
            {
                string line;
                vector<string> tokens;
                getline (F1,line);
                
                
                string currentmarker = "";
                int n = Tokenize(string(line), tokens, " ");		//tabulating file by space
                if (n>2)
                {
                    int chr;
                    if (uc(tokens[0])=="MT") chr=26;
                    else if (uc(tokens[0])=="XY") chr=25;
                    else if (uc(tokens[0])=="Y") chr=24;
                    else if (uc(tokens[0])=="X") chr=23;
                    else chr = atoi(tokens[0].c_str());
                    
                    if (G.debugMode) cout << "CONDITIONAL MARKER MARKER POS: Chromosome id: " << chr;
                    
                    if (forcedChromosome) 
                    {
                        chr = forcedChromosome;
                        if (G.debugMode) cout << "CONDITIONAL MARKER POS: Chromosome forced: " << chr;                    
                    }
                    string markerPos = string(tokens[2]);
                    string markerName = string(tokens[1]);
                    char effectAllele = tokens[3][0];
                    char nonEffectAllele = tokens[4][0];
                    if (G.debugMode) cout << "CONDITIONAL MARKER POS: Pos: " << markerPos << "\nmarker:" << markerName << "\nea/nea:" << effectAllele <<"/" << nonEffectAllele<<"\n";
                    
                    vector <double> gen;
                    vector <double> gen2;
                    //                double fijeij = 0;                 //for info score
                    double aa=0; double aA=0; double AA=0;
                    double callrate=0; double ok_gen=0; double not_ok_gen=0;
                    double infoscore = 1;
                    int j = 0;
                    for (int i = 5; i < n; i+=3)
                    {
                        //determined
                        if (S[j].isOK())
                        {
                            if(G.dosages==false)
                            {
                                if (atof(tokens[i].c_str())>=.95){aa++;}
                                else if (atof(tokens[i+1].c_str())>=.95){aA++;}
                                else if (atof(tokens[i+2].c_str())>=.95){AA++;}
                                
                                if (atof(tokens[i].c_str())>=.95){gen.push_back(2.0);ok_gen++;}
                                else if (atof(tokens[i+1].c_str())>=.95){gen.push_back(1.0);ok_gen++;}
                                else if (atof(tokens[i+2].c_str())>=.95){gen.push_back(0.0);ok_gen++;}
                                else {gen.push_back(-9999);not_ok_gen++;}
                            }
                            else
                            {
                                aa+=atof(tokens[i].c_str());
                                aA+=atof(tokens[i+1].c_str());
                                AA+=atof(tokens[i+2].c_str());
                                gen2.push_back(((atof(tokens[i+2].c_str())) + (atof(tokens[i+1].c_str())/2)));
                                gen.push_back(((atof(tokens[i].c_str())) + (atof(tokens[i+1].c_str())/2)));
                            }
                        }
                        if (n < i+5) i+=3;		//avoiding space in line end
                        
                        j++;
                    }
                    if (G.debugMode)cout<<"CONDITIONAL MARKER POS: aa,aA,AA"<< aa<<","<<aA<<","<<AA<<"\n";
                    if (G.dosages)
                    {
                        callrate=1;
                        double eaf = 0;
                        if (aa+aA+AA>0)
                        {
                            eaf = ((2*aa)+aA)/(2*(aa+aA+AA));
                            
                        }
                        if (eaf==1 || eaf==0){infoscore=1;}
                        //info score calculation according to the measure in snptest:             
                        //https://mathgen.stats.ox.ac.uk/genetics_software/snptest/snptest.v2.pdf
                        
                    }
                    else 
                    {
                        callrate = ok_gen/(ok_gen+not_ok_gen);   
                        if (G.debugMode)cout<<"CONDITIONAL MARKER POS: Callrate: "<< callrate <<endl;
                    }
                    
                    std::stringstream mrk2;
                    mrk2 << chr << ":" << markerPos; 
                    string mrkname = mrk2.str();
                    
                    if (aa+aA+AA>0 && markerPos == marker)
                    {
                        _markerPos = gen;
                        return true;
                    }
                }
            }
        }
        else 
        {
            cerr << "Error: Cannot find or access " << G.inputGenFile << ". Exit program." << endl;
            sprintf(sb, "E%.8d" , G.errNr); G.errNr++;
            LOG << "Error: Cannot find or access " << G.inputGenFile << ". Exit program! (" << sb << ")" <<endl;
            ERR << sb << " Error: Cannot find or access " << G.inputGenFile << ". Program exit!" <<endl;
            ERR << sb << " Error: Make sure that this file is in given path and is readable." <<endl;
            return false;
        }
        F1.close();
    }
    cerr << "Error: Cannot find conditional marker pos " << marker << ". Exit program." << endl;
    sprintf(sb, "E%.8d" , G.errNr); G.errNr++;
    LOG << "Error: Cannot find conditional marker " << marker << ". Exit program! (" << sb << ")" <<endl;
    ERR << sb << " Error: Cannot find conditional marker " << marker << ". Program exit!" <<endl;
    exit (1);
    return false;
}


bool
readConditionalData(global & G,  vector <sample> & S,ofstream & LOG,ofstream & ERR, gene  & GENE_p)
{
	char sb [10];				//error name char buffer
	int m_ok = 0;
	int m_notok=0;
	int forcedChromosome = G.chromosome; // I am forcing all markers from the input file to be be from predefined chromosome - snptest has "---" in first column for imputed snps instead of chromosome number
    if (G.inputGenFile.substr(G.inputGenFile.length()-2)=="gz")
    {
        //      	ifstream F (G.inputGenFile.c_str());
        gzFile F =gzopen(G.inputGenFile.c_str(),"r");
        char *buffer = new char[LENS];
        while(0!=gzgets(F,buffer,LENS))
        {
            string line;
            vector<string> tokens;
            string currentmarker = "";
            int n = Tokenize(buffer, tokens, " ");		//tabulating file by space
            if (n>2)
            {
                int chr;
                if (uc(tokens[0])=="MT") chr=26;
                else if (uc(tokens[0])=="XY") chr=25;
                else if (uc(tokens[0])=="Y") chr=24;
                else if (uc(tokens[0])=="X") chr=23;
                else chr = atoi(tokens[0].c_str());
                
                if (G.debugMode) cout << "CONDITIONAL: Chromosome id: " << chr;
                
                if (forcedChromosome) 
                {
                    chr = forcedChromosome;
                    if (G.debugMode) cout << "CONDITIONAL: Chromosome forced: " << chr;                    
                }
                int pos = atoi(tokens[2].c_str());
                string markerName = string(tokens[1]);
                char effectAllele = tokens[3][0];
                char nonEffectAllele = tokens[4][0];
                if (G.debugMode) cout << "CONDITIONAL: Pos: " << pos << "\nmarker:" << markerName << "\nea/nea:" << effectAllele <<"/" << nonEffectAllele<<"\n";
                
                vector <double> gen;
                vector <double> gen2;
                double fijeij = 0;                 //for info score
                double aa=0; double aA=0; double AA=0;
                double callrate=0; double ok_gen=0; double not_ok_gen=0;
                double infoscore = 1;
                int j = 0;
                for (int i = 5; i < n; i+=3)
                {
                    //determined
                    if (S[j].isOK())
                    {
                        if(G.dosages==false)
                        {
                            if (atof(tokens[i].c_str())>=.95){aa++;}
                            else if (atof(tokens[i+1].c_str())>=.95){aA++;}
                            else if (atof(tokens[i+2].c_str())>=.95){AA++;}
                            
                            if (atof(tokens[i].c_str())>=.95){gen.push_back(2.0);ok_gen++;}
                            else if (atof(tokens[i+1].c_str())>=.95){gen.push_back(1.0);ok_gen++;}
                            else if (atof(tokens[i+2].c_str())>=.95){gen.push_back(0.0);ok_gen++;}
                            else {gen.push_back(-9999);not_ok_gen++;}
                        }
                        else
                        {
                            aa+=atof(tokens[i].c_str());
                            aA+=atof(tokens[i+1].c_str());
                            AA+=atof(tokens[i+2].c_str());
                            double eij=(2*atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str());
                            double fij = (4*atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str());
                            fijeij+= fij - (eij*eij);
                            
                            gen2.push_back(((atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str())));
                            gen.push_back(((atof(tokens[i].c_str())) + atof(tokens[i+1].c_str())));
                        }
                    }
                    if (n < i+5) i+=3;		//avoiding space in line end
                    
                    j++;
                }
                if (G.debugMode)cout<<"CONDITIONAL: aa,aA,AA"<< aa<<","<<aA<<","<<AA<<"\n";
                if (G.dosages)
                {
                    callrate=1;
                    double eaf = 0;
                    if (aa+aA+AA>0)
                    {
                        eaf = ((2*aa)+aA)/(2*(aa+aA+AA));
                        
                    }
                    if (eaf==1 || eaf==0){infoscore=1;}
                    //info score calculation according to the measure in snptest:             
                    //https://mathgen.stats.ox.ac.uk/genetics_software/snptest/snptest.v2.pdf
                    else                                  
                    {   
                        infoscore = 1-(fijeij/(2*(aa+aA+AA)*eaf*(1-eaf)));
                    }
                    if (G.debugMode)cout<<"CONDITIONAL: N: " << n << " eaf: " << eaf << " aa+aA+AA: " << aa+aA+AA << " Infoscore: "<< infoscore<<endl;
                    
                }
                else 
                {
                    callrate = ok_gen/(ok_gen+not_ok_gen);   
                    if (G.debugMode)cout<<"CONDITIONAL: Callrate: "<< callrate <<endl;
                }
                
                std::stringstream mrk2;
                mrk2 << chr << ":" << pos; 
            std:string mrkname = mrk2.str();
                
                bool markerExclusionInclusionIsOK=true;
                
                if (G.markerExclusion)
                {
                    if (G.excludemarkers[mrkname]){markerExclusionInclusionIsOK=false;}
                }
                if (G.markerExtraction)
                {
                    if (!G.extractmarkers[mrkname]){markerExclusionInclusionIsOK=false;}
                }
                
                if (aa+aA+AA>0 && markerExclusionInclusionIsOK)
                {
                    if (((2*aa)+aA)/(2*(aa+aA+AA))<=0.5)
                    {
                        marker m(markerName, effectAllele, nonEffectAllele, gen);
                        m.setMaf(((2*aa)+aA)/(2*(aa+aA+AA)));
                        
                        
                        if (pos>0 && chr>0 && chr<27 && G.rareVariantThreshold>=m.getMaf() && G.callrateThreshold<=callrate && G.infoThreshold<=infoscore && m.getMaf()>0 && chr == GENE_p.chr && pos>=GENE_p.start && pos<=GENE_p.end)
                        {
                            m.setChromosome(chr);
                            m.setPosition(pos);
                            GENE_p.M.push_back(m);
                            m_ok++;
                            if (G.debugMode)cout<<"CONDITIONAL: Marker added to array with maf: "<< ((2*aa)+aA)/(2*(aa+aA+AA))<<"\n";
                            if (G.debugMode)cout<<"CONDITIONAL: Rare allele: "<< effectAllele <<"\n";
                        }
                        else
                        {
                            m_notok++;
                            if (G.debugMode)cout<<"CONDITIONAL: Marker not added\n";
                        }
                    }
                    else
                    {
                        for (unsigned int j = 0; j < gen.size(); j++)
                        {
                            if (!G.dosages){if (gen[j]==2){gen[j]=0;} else if (gen[j]==0){gen[j]=2;}}
                            else {gen[j] = gen2[j];}
                            
                        } //turning around
                        marker m(markerName, nonEffectAllele, effectAllele, gen);
                        m.setMaf(((2*AA)+aA)/(2*(aa+aA+AA)));
                        if (pos>0 && chr>0 && chr<27 && G.rareVariantThreshold>=m.getMaf() && G.callrateThreshold<=callrate && G.infoThreshold<=infoscore &&m.getMaf()>0 && chr == GENE_p.chr && pos>=GENE_p.start && pos<=GENE_p.end)
                        {
                            m.setChromosome(chr);
                            m.setPosition(pos);
                            GENE_p.M.push_back(m);
                            m_ok++;
                            if (G.debugMode)cout<<"CONDITIONAL: Marker added to array with maf: "<< ((2*AA)+aA)/(2*(aa+aA+AA))<<"\n";
                            if (G.debugMode)cout<<"CONDITIONAL: Rare allele: "<< nonEffectAllele <<"\n";
                        }
                        else
                        {
                            m_notok++;
                            if (G.debugMode)cout<<"CONDITIONAL: Marker not added\n";
                        }
                    }
                }
            }
        }
        cout << "Markers added from conditional gene region: "<< m_ok << endl;
        //		cout << "Markers removed "<< m_notok << endl;
        
    }
    else
    {
        ifstream F1 (G.inputGenFile.c_str());
        if (F1.is_open())
        {
            while (! F1.eof() )
            {
                string line;
                vector<string> tokens;
                getline (F1,line);
                string currentmarker = "";
                int n = Tokenize(string(line), tokens, " ");		//tabulating file by space
                if (n>2)
                {
                    int chr;
                    if (uc(tokens[0])=="MT") chr=26;
                    else if (uc(tokens[0])=="XY") chr=25;
                    else if (uc(tokens[0])=="Y") chr=24;
                    else if (uc(tokens[0])=="X") chr=23;
                    else chr = atoi(tokens[0].c_str());
                    
                    if (G.debugMode) cout << "CONDITIONAL: Chromosome id: " << chr;
                    
                    if (forcedChromosome) 
                    {
                        chr = forcedChromosome;
                        if (G.debugMode) cout << "CONDITIONAL: Chromosome forced: " << chr;                    
                    }
                    int pos = atoi(tokens[2].c_str());
                    string markerName = string(tokens[1]);
                    char effectAllele = tokens[3][0];
                    char nonEffectAllele = tokens[4][0];
                    if (G.debugMode) cout << "CONDITIONAL: Pos: " << pos << "\nmarker:" << markerName << "\nea/nea:" << effectAllele <<"/" << nonEffectAllele<<"\n";
                    
                    vector <double> gen;
                    vector <double> gen2;
                    double fijeij = 0;                 //for info score
                    double aa=0; double aA=0; double AA=0;
                    double callrate=0; double ok_gen=0; double not_ok_gen=0;
                    double infoscore = 1;
                    int j = 0;
                    for (int i = 5; i < n; i+=3)
                    {
                        //determined
                        if (S[j].isOK())
                        {
                        if(G.dosages==false)
                        {
                            if (atof(tokens[i].c_str())>=.95){aa++;}
                            else if (atof(tokens[i+1].c_str())>=.95){aA++;}
                            else if (atof(tokens[i+2].c_str())>=.95){AA++;}

                            if (atof(tokens[i].c_str())>=.95){gen.push_back(2.0);ok_gen++;}
                            else if (atof(tokens[i+1].c_str())>=.95){gen.push_back(1.0);ok_gen++;}
                            else if (atof(tokens[i+2].c_str())>=.95){gen.push_back(0.0);ok_gen++;}
                            else {gen.push_back(-9999);not_ok_gen++;}
                        }
                        else
                        {
                            aa+=atof(tokens[i].c_str());
                            aA+=atof(tokens[i+1].c_str());
                            AA+=atof(tokens[i+2].c_str());
                            double eij=(2*atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str());
                            double fij = (4*atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str());
                            fijeij+= fij - (eij*eij);

                            gen2.push_back(((atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str())));
                            gen.push_back(((atof(tokens[i].c_str())) + atof(tokens[i+1].c_str())));
                        }
                        }
                        if (n < i+5) i+=3;		//avoiding space in line end
                        
                        j++;
                    }
                    if (G.debugMode)cout<<"CONDITIONAL: aa,aA,AA"<< aa<<","<<aA<<","<<AA<<"\n";
                    if (G.dosages)
                    {
                        callrate=1;
                        double eaf = 0;
                        if (aa+aA+AA>0)
                        {
                            eaf = ((2*aa)+aA)/(2*(aa+aA+AA));
                            
                        }
                        if (eaf==1 || eaf==0){infoscore=1;}
                        //info score calculation according to the measure in snptest:             
                        //https://mathgen.stats.ox.ac.uk/genetics_software/snptest/snptest.v2.pdf
                        else                                  
                        {   
                            infoscore = 1-(fijeij/(2*(aa+aA+AA)*eaf*(1-eaf)));
                        }
                        if (G.debugMode)cout<<"CONDITIONAL: N: " << n << " eaf: " << eaf << " aa+aA+AA: " << aa+aA+AA << " Infoscore: "<< infoscore<<endl;
                        
                    }
                    else 
                    {
                        callrate = ok_gen/(ok_gen+not_ok_gen);   
                        if (G.debugMode)cout<<"CONDITIONAL: Callrate: "<< callrate <<endl;
                    }
                    
                    std::stringstream mrk2;
                    mrk2 << chr << ":" << pos; 
                    string mrkname = mrk2.str();
                    
                    bool markerExclusionInclusionIsOK=true;
                    
                    if (G.markerExclusion)
                    {
                        if (G.excludemarkers[mrkname]){markerExclusionInclusionIsOK=false;}
                    }
                    if (G.markerExtraction)
                    {
                        if (!G.extractmarkers[mrkname]){markerExclusionInclusionIsOK=false;}
                    }
                    
                    if (aa+aA+AA>0 && markerExclusionInclusionIsOK)
                    {
                        if (((2*aa)+aA)/(2*(aa+aA+AA))<=0.5)
                        {
                            marker m(markerName, effectAllele, nonEffectAllele, gen);
                            m.setMaf(((2*aa)+aA)/(2*(aa+aA+AA)));
                            
                                                
                            if (pos>0 && chr>0 && chr<27 && G.rareVariantThreshold>=m.getMaf() && G.callrateThreshold<=callrate && G.infoThreshold<=infoscore && m.getMaf()>0 && chr == GENE_p.chr && pos>=GENE_p.start && pos<=GENE_p.end)
                            {
                                m.setChromosome(chr);
                                m.setPosition(pos);
                                GENE_p.M.push_back(m);
                                m_ok++;
                                if (G.debugMode)cout<<"CONDITIONAL: Marker added to array with maf: "<< ((2*aa)+aA)/(2*(aa+aA+AA))<<"\n";
                                if (G.debugMode)cout<<"CONDITIONAL: Rare allele: "<< effectAllele <<"\n";
                            }
                            else
                            {
                                m_notok++;
                                if (G.debugMode)cout<<"CONDITIONAL: Marker not added\n";
                            }
                        }
                        else
                        {
                            for (unsigned int j = 0; j < gen.size(); j++)
                            {
                                if (!G.dosages){if (gen[j]==2){gen[j]=0;} else if (gen[j]==0){gen[j]=2;}}
                                else {gen[j] = gen2[j];}
                                
                            } //turning around
                            marker m(markerName, nonEffectAllele, effectAllele, gen);
                            m.setMaf(((2*AA)+aA)/(2*(aa+aA+AA)));
                            if (pos>0 && chr>0 && chr<27 && G.rareVariantThreshold>=m.getMaf() && G.callrateThreshold<=callrate && G.infoThreshold<=infoscore &&m.getMaf()>0 && chr == GENE_p.chr && pos>=GENE_p.start && pos<=GENE_p.end)
                            {
                                m.setChromosome(chr);
                                m.setPosition(pos);
                                GENE_p.M.push_back(m);
                                m_ok++;
                                if (G.debugMode)cout<<"CONDITIONAL: Marker added to array with maf: "<< ((2*AA)+aA)/(2*(aa+aA+AA))<<"\n";
                                if (G.debugMode)cout<<"CONDITIONAL: Rare allele: "<< nonEffectAllele <<"\n";
                            }
                            else
                            {
                                m_notok++;
                                if (G.debugMode)cout<<"CONDITIONAL: Marker not added\n";
                            }
                        }
                    }
                }
            }
            cout << "Markers added from conditional gene region: "<< m_ok << endl;
    //		cout << "Markers removed "<< m_notok << endl;
        }
        else 
        {
            cerr << "Error: Cannot find or access " << G.inputGenFile << ". Exit program." << endl;
            sprintf(sb, "E%.8d" , G.errNr); G.errNr++;
            LOG << "Error: Cannot find or access " << G.inputGenFile << ". Exit program! (" << sb << ")" <<endl;
            ERR << sb << " Error: Cannot find or access " << G.inputGenFile << ". Program exit!" <<endl;
            ERR << sb << " Error: Make sure that this file is in given path and is readable." <<endl;
            return false;
        }
        F1.close();
    }
	return true;
}

//
// SORTED LOW MEM
//
bool 
readGenotypeData(global & G, vector <sample> & S,  vector <gene> & GENE,  ofstream & LOG, ofstream & ERR, ofstream & OUT)
{
	char sb [10];				//error name char buffer
	int m_ok = 0;
	int m_notok=0;
	int forcedChromosome = G.chromosome; // I am forcing all markers from the input file to be be from predefined chromosome - snptest has "---" in first column for imputed snps instead of chromosome number
    
//    LowMemPrintGene(G, S, GENE_p, LOG, ERR, OUT);
    if (G.inputGenFile.substr(G.inputGenFile.length()-2)=="gz")
    {
//      	ifstream F (G.inputGenFile.c_str());
        gzFile F =gzopen(G.inputGenFile.c_str(),"r");
            char *buffer = new char[LENS];
            while(0!=gzgets(F,buffer,LENS))
            {
                string line;
                vector<string> tokens;
                string currentmarker = "";
                int n = Tokenize(buffer, tokens, " ");		//tabulating file by space
                if (n>2)
                {
                    int chr;
                    if (uc(tokens[0])=="MT") chr=26;
                    else if (uc(tokens[0])=="XY") chr=25;
                    else if (uc(tokens[0])=="Y") chr=24;
                    else if (uc(tokens[0])=="X") chr=23;
                    else chr = atoi(tokens[0].c_str());
                    
                    if (G.debugMode) cout << "Chromosome id: " << chr;
                    
                    if (forcedChromosome) 
                    {
                        chr = forcedChromosome;
                        if (G.debugMode) cout << "Chromosome forced: " << chr;                    
                    }
                    int pos = atoi(tokens[2].c_str());
                    string markerName = string(tokens[1]);
                    char effectAllele = tokens[3][0];
                    char nonEffectAllele = tokens[4][0];
                    if (G.debugMode) cout << "Pos: " << pos << "\nmarker:" << markerName << "\nea/nea:" << effectAllele <<"/" << nonEffectAllele<<"\n";
                    
                    vector <double> gen;
                    vector <double> gen2;
                    double fijeij = 0;                 //for info score
                    double aa=0; double aA=0; double AA=0;
                    double callrate=0; double ok_gen=0; double not_ok_gen=0;
                    double infoscore = 1;
                    int j = 0;
                    for (int i = 5; i < n; i+=3)
                    {
                        //determined
                        if(G.dosages==false)
                        {
                            if (S[j].isOK())
                            {
                                if (atof(tokens[i].c_str())>=.95){aa++;}
                                else if (atof(tokens[i+1].c_str())>=.95){aA++;}
                                else if (atof(tokens[i+2].c_str())>=.95){AA++;}
                            }
                            if (atof(tokens[i].c_str())>=.95){gen.push_back(2.0);ok_gen++;}
                            else if (atof(tokens[i+1].c_str())>=.95){gen.push_back(1.0);ok_gen++;}
                            else if (atof(tokens[i+2].c_str())>=.95){gen.push_back(0.0);ok_gen++;}
                            else {gen.push_back(-9999);not_ok_gen++;}
                        }
                        else
                        {
                            if (S[j].isOK())
                            {
                                aa+=atof(tokens[i].c_str());
                                aA+=atof(tokens[i+1].c_str());
                                AA+=atof(tokens[i+2].c_str());
                                double eij=(2*atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str());
                                double fij = (4*atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str());
                                fijeij+= fij - (eij*eij);
                            }
                            gen2.push_back(((atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str())));
                            gen.push_back(((atof(tokens[i].c_str())) + atof(tokens[i+1].c_str())));
                        }
                        if (n < i+5) i+=3;		//avoiding space in line end
                        
                        j++;
                    }
                    if (G.debugMode)cout<<"aa,aA,AA"<< aa<<","<<aA<<","<<AA<<"\n";
                    if (G.dosages)
                    {
                        callrate=1;
                        double eaf = 0;
                        if (aa+aA+AA>0)
                        {
                            eaf = ((2*aa)+aA)/(2*(aa+aA+AA));
                            
                        }
                        if (eaf==1 || eaf==0){infoscore=1;}
                        //info score calculation according to the measure in snptest:             
                        //https://mathgen.stats.ox.ac.uk/genetics_software/snptest/snptest.v2.pdf
                        else                                  
                        {   
                            infoscore = 1-(fijeij/(2*(aa+aA+AA)*eaf*(1-eaf)));
                        }
                        if (G.debugMode)cout<<"N: " << n << " eaf: " << eaf << " aa+aA+AA: " << aa+aA+AA << " Infoscore: "<< infoscore<<endl;
                        
                    }
                    else 
                    {
                        callrate = ok_gen/(ok_gen+not_ok_gen);   
                        if (G.debugMode)cout<<"Callrate: "<< callrate <<endl;
                    }
                    
                    std::stringstream mrk2;
                    mrk2 << chr << ":" << pos; 
                std:string mrkname = mrk2.str();
                    
                    bool markerExclusionInclusionIsOK=true;
                    
                    if (G.markerExclusion)
                    {
                        if (G.excludemarkers[mrkname]){markerExclusionInclusionIsOK=false;}
                    }
                    if (G.markerExtraction)
                    {
                        if (!G.extractmarkers[mrkname]){markerExclusionInclusionIsOK=false;}
                    }
                    
                    if (aa+aA+AA>0 && markerExclusionInclusionIsOK)
                    {
                        if (((2*aa)+aA)/(2*(aa+aA+AA))<=0.5)
                        {
                            marker m(markerName, effectAllele, nonEffectAllele, gen);
                            m.setMaf(((2*aa)+aA)/(2*(aa+aA+AA)));
                            if (pos>0 && chr>0 && chr<27 && G.rareVariantThreshold>=m.getMaf() && G.callrateThreshold<=callrate && G.infoThreshold<=infoscore && m.getMaf()>0)
                            {
                                m.setChromosome(chr);
                                m.setPosition(pos);
                                for (int zz=0; zz<GENE.size(); zz++)
                                {
                                    if (chr == GENE[zz].chr &&
                                        pos>= GENE[zz].start &&
                                        pos<=  GENE[zz].end)
                                    {
                                        GENE[zz].M.push_back(m);
                                    }
                                    else if (chr == GENE[zz].chr && pos>GENE[zz].end && GENE[zz].M.size()>0)
                                    {
                                        LowMemPrintGene(G, S, GENE[zz], LOG, ERR, OUT);
                                        GENE[zz].M.clear();
                                    }
                                }
                                m_ok++;
                                if (G.debugMode)cout<<"Marker added to array with maf: "<< ((2*aa)+aA)/(2*(aa+aA+AA))<<"\n";
                                if (G.debugMode)cout<<"Rare allele: "<< effectAllele <<"\n";
                            }
                            else
                            {
                                m_notok++;
                                if (G.debugMode)cout<<"Marker not added\n";
                            }
                        }
                        else
                        {
                            for (unsigned int j = 0; j < gen.size(); j++)
                            {
                                if (!G.dosages){if (gen[j]==2){gen[j]=0;} else if (gen[j]==0){gen[j]=2;}}
                                else {gen[j] = gen2[j];}
                                
                            } //turning around
                            marker m(markerName, nonEffectAllele, effectAllele, gen);
                            m.setMaf(((2*AA)+aA)/(2*(aa+aA+AA)));
                            if (pos>0 && chr>0 && chr<27 && G.rareVariantThreshold>=m.getMaf() && G.callrateThreshold<=callrate && G.infoThreshold<=infoscore && m.getMaf()>0)
                            {
                                m.setChromosome(chr);
                                m.setPosition(pos);
                                
                                for (int zz=0; zz<GENE.size(); zz++)
                                {
                                    if (chr == GENE[zz].chr &&
                                        pos>= GENE[zz].start &&
                                        pos<=  GENE[zz].end)
                                    {
                                        GENE[zz].M.push_back(m);
                                    }
                                    else if (chr == GENE[zz].chr && pos>GENE[zz].end && GENE[zz].M.size()>0)
                                    {
                                        LowMemPrintGene(G, S, GENE[zz], LOG, ERR, OUT);
                                        GENE[zz].M.clear();
                                    }
                                }
                                
                                m_ok++;
                                if (G.debugMode)cout<<"Marker added to array with maf: "<< ((2*AA)+aA)/(2*(aa+aA+AA))<<"\n";
                                if (G.debugMode)cout<<"Rare allele: "<< nonEffectAllele <<"\n";
                            }
                            else
                            {
                                m_notok++;
                                if (G.debugMode)cout<<"Marker not added\n";
                            }
                        }
                    }
                }
            }
            for (int zz=0; zz<GENE.size(); zz++)
            {
                if (GENE[zz].M.size()>0)
                {
                    LowMemPrintGene(G, S, GENE[zz], LOG, ERR, OUT);
                    GENE[zz].M.clear();
                }
            }
            

        return true;
  
    }
    else
    {
    
        ifstream F (G.inputGenFile.c_str());
        if (F.is_open())
        {
            while (! F.eof() )
            {
                string line;
                vector<string> tokens;
                getline (F,line);
                string currentmarker = "";
                int n = Tokenize(string(line), tokens, " ");		//tabulating file by space
                if (n>2)
                {
                    int chr;
                    if (uc(tokens[0])=="MT") chr=26;
                    else if (uc(tokens[0])=="XY") chr=25;
                    else if (uc(tokens[0])=="Y") chr=24;
                    else if (uc(tokens[0])=="X") chr=23;
                    else chr = atoi(tokens[0].c_str());
                    
                    if (G.debugMode) cout << "Chromosome id: " << chr;
                    
                    if (forcedChromosome) 
                    {
                        chr = forcedChromosome;
                        if (G.debugMode) cout << "Chromosome forced: " << chr;                    
                    }
                    int pos = atoi(tokens[2].c_str());
                    string markerName = string(tokens[1]);
                    char effectAllele = tokens[3][0];
                    char nonEffectAllele = tokens[4][0];
                    if (G.debugMode) cout << "Pos: " << pos << "\nmarker:" << markerName << "\nea/nea:" << effectAllele <<"/" << nonEffectAllele<<"\n";
                    
                    vector <double> gen;
                    vector <double> gen2;
                    double fijeij = 0;                 //for info score
                    double aa=0; double aA=0; double AA=0;
                    double callrate=0; double ok_gen=0; double not_ok_gen=0;
                    double infoscore = 1;
                    int j = 0;
                    for (int i = 5; i < n; i+=3)
                    {
                        //determined
                        if(G.dosages==false)
                        {
                            if (S[j].isOK())
                            {
                                if (atof(tokens[i].c_str())>=.95){aa++;}
                                else if (atof(tokens[i+1].c_str())>=.95){aA++;}
                                else if (atof(tokens[i+2].c_str())>=.95){AA++;}
                            }
                            if (atof(tokens[i].c_str())>=.95){gen.push_back(2.0);ok_gen++;}
                            else if (atof(tokens[i+1].c_str())>=.95){gen.push_back(1.0);ok_gen++;}
                            else if (atof(tokens[i+2].c_str())>=.95){gen.push_back(0.0);ok_gen++;}
                            else {gen.push_back(-9999);not_ok_gen++;}
                        }
                        else
                        {
                            if (S[j].isOK())
                            {
                                aa+=atof(tokens[i].c_str());
                                aA+=atof(tokens[i+1].c_str());
                                AA+=atof(tokens[i+2].c_str());
                                double eij=(2*atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str());
                                double fij = (4*atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str());
                                fijeij+= fij - (eij*eij);
                            }
                            gen2.push_back(((atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str())));
                            gen.push_back(((atof(tokens[i].c_str())) + atof(tokens[i+1].c_str())));
                        }
                        if (n < i+5) i+=3;		//avoiding space in line end
                        
                        j++;
                    }
                    if (G.debugMode)cout<<"aa,aA,AA"<< aa<<","<<aA<<","<<AA<<"\n";
                    if (G.dosages)
                    {
                        callrate=1;
                        double eaf = 0;
                        if (aa+aA+AA>0)
                        {
                            eaf = ((2*aa)+aA)/(2*(aa+aA+AA));
                            
                        }
                        if (eaf==1 || eaf==0){infoscore=1;}
                        //info score calculation according to the measure in snptest:             
                        //https://mathgen.stats.ox.ac.uk/genetics_software/snptest/snptest.v2.pdf
                        else                                  
                        {   
                            infoscore = 1-(fijeij/(2*(aa+aA+AA)*eaf*(1-eaf)));
                        }
                        if (G.debugMode)cout<<"N: " << n << " eaf: " << eaf << " aa+aA+AA: " << aa+aA+AA << " Infoscore: "<< infoscore<<endl;
                        
                    }
                    else 
                    {
                        callrate = ok_gen/(ok_gen+not_ok_gen);   
                        if (G.debugMode)cout<<"Callrate: "<< callrate <<endl;
                    }
                    
                    std::stringstream mrk2;
                    mrk2 << chr << ":" << pos; 
                    string mrkname = mrk2.str();
                    
                    bool markerExclusionInclusionIsOK=true;
                    
                    if (G.markerExclusion)
                    {
                        if (G.excludemarkers[mrkname]){markerExclusionInclusionIsOK=false;}
                    }
                    if (G.markerExtraction)
                    {
                        if (!G.extractmarkers[mrkname]){markerExclusionInclusionIsOK=false;}
                    }
                    
                    if (aa+aA+AA>0 && markerExclusionInclusionIsOK)
                    {
                        if (((2*aa)+aA)/(2*(aa+aA+AA))<=0.5)
                        {
                            marker m(markerName, effectAllele, nonEffectAllele, gen);
                            m.setMaf(((2*aa)+aA)/(2*(aa+aA+AA)));
                            if (pos>0 && chr>0 && chr<27 && G.rareVariantThreshold>=m.getMaf() && G.callrateThreshold<=callrate && G.infoThreshold<=infoscore && m.getMaf()>0)
                            {
                                m.setChromosome(chr);
                                m.setPosition(pos);
                                for (int zz=0; zz<GENE.size(); zz++)
                                {
                                    if (chr == GENE[zz].chr &&
                                        pos>= GENE[zz].start &&
                                        pos<=  GENE[zz].end)
                                    {
                                        GENE[zz].M.push_back(m);
                                    }
                                    else if (chr == GENE[zz].chr && pos>GENE[zz].end && GENE[zz].M.size()>0)
                                    {
                                        LowMemPrintGene(G, S, GENE[zz], LOG, ERR, OUT);
                                        GENE[zz].M.clear();
                                    }
                                }
                                m_ok++;
                                if (G.debugMode)cout<<"Marker added to array with maf: "<< ((2*aa)+aA)/(2*(aa+aA+AA))<<"\n";
                                if (G.debugMode)cout<<"Rare allele: "<< effectAllele <<"\n";
                            }
                            else
                            {
                                m_notok++;
                                if (G.debugMode)cout<<"Marker not added\n";
                            }
                        }
                        else
                        {
                            for (unsigned int j = 0; j < gen.size(); j++)
                            {
                                if (!G.dosages){if (gen[j]==2){gen[j]=0;} else if (gen[j]==0){gen[j]=2;}}
                                else {gen[j] = gen2[j];}
                                
                            } //turning around
                            marker m(markerName, nonEffectAllele, effectAllele, gen);
                            m.setMaf(((2*AA)+aA)/(2*(aa+aA+AA)));
                            if (pos>0 && chr>0 && chr<27 && G.rareVariantThreshold>=m.getMaf() && G.callrateThreshold<=callrate && G.infoThreshold<=infoscore && m.getMaf()>0)
                            {
                                m.setChromosome(chr);
                                m.setPosition(pos);
                                
                                for (int zz=0; zz<GENE.size(); zz++)
                                {
                                    if (chr == GENE[zz].chr &&
                                        pos>= GENE[zz].start &&
                                        pos<=  GENE[zz].end)
                                    {
                                        GENE[zz].M.push_back(m);
                                    }
                                    else if (chr == GENE[zz].chr && pos>GENE[zz].end && GENE[zz].M.size()>0)
                                    {
                                        LowMemPrintGene(G, S, GENE[zz], LOG, ERR, OUT);
                                        GENE[zz].M.clear();
                                    }
                                }
                                
                                m_ok++;
                                if (G.debugMode)cout<<"Marker added to array with maf: "<< ((2*AA)+aA)/(2*(aa+aA+AA))<<"\n";
                                if (G.debugMode)cout<<"Rare allele: "<< nonEffectAllele <<"\n";
                            }
                            else
                            {
                                m_notok++;
                                if (G.debugMode)cout<<"Marker not added\n";
                            }
                        }
                    }
                }
            }
            for (int zz=0; zz<GENE.size(); zz++)
            {
                if (GENE[zz].M.size()>0)
                {
                    LowMemPrintGene(G, S, GENE[zz], LOG, ERR, OUT);
                    GENE[zz].M.clear();
                }
            }
            
        }
        else 
        {
            cerr << "Error: Cannot find or access " << G.inputGenFile << ". Exit program." << endl;
            sprintf(sb, "E%.8d" , G.errNr); G.errNr++;
            LOG << "Error: Cannot find or access " << G.inputGenFile << ". Exit program! (" << sb << ")" <<endl;
            ERR << sb << " Error: Cannot find or access " << G.inputGenFile << ". Program exit!" <<endl;
            ERR << sb << " Error: Make sure that this file is in given path and is readable." <<endl;
            return false;
        }
        return true;
    }
}

//
//
bool 
readGenotypeData(global & G, vector <sample> & S,  vector <gene> & GENE,  ofstream & LOG, ofstream & ERR, ofstream & OUTM, ofstream & OUTF)
{
	char sb [10];				//error name char buffer
	int m_ok = 0;
	int m_notok=0;
	int forcedChromosome = G.chromosome; // I am forcing all markers from the input file to be be from predefined chromosome - snptest has "---" in first column for imputed snps instead of chromosome number
    
    //    LowMemPrintGene(G, S, GENE_p, LOG, ERR, OUT);
    if (G.inputGenFile.substr(G.inputGenFile.length()-2)=="gz")
    {
        //      	ifstream F (G.inputGenFile.c_str());
        gzFile F =gzopen(G.inputGenFile.c_str(),"r");
        char *buffer = new char[LENS];
        while(0!=gzgets(F,buffer,LENS))
            {
                string line;
                vector<string> tokens;

                string currentmarker = "";
                int n = Tokenize(buffer, tokens, " ");		//tabulating file by space
                if (n>2)
                {
                    int chr;
                    if (uc(tokens[0])=="MT") chr=26;
                    else if (uc(tokens[0])=="XY") chr=25;
                    else if (uc(tokens[0])=="Y") chr=24;
                    else if (uc(tokens[0])=="X") chr=23;
                    else chr = atoi(tokens[0].c_str());
                    
                    if (G.debugMode) cout << "Chromosome id: " << chr;
                    
                    if (forcedChromosome) 
                    {
                        chr = forcedChromosome;
                        if (G.debugMode) cout << "Chromosome forced: " << chr;                    
                    }
                    int pos = atoi(tokens[2].c_str());
                    string markerName = string(tokens[1]);
                    char effectAllele = tokens[3][0];
                    char nonEffectAllele = tokens[4][0];
                    if (G.debugMode) cout << "Pos: " << pos << "\nmarker:" << markerName << "\nea/nea:" << effectAllele <<"/" << nonEffectAllele<<"\n";
                    
                    vector <double> gen;
                    vector <double> gen2;
                    double fijeij = 0;                 //for info score
                    double aa=0; double aA=0; double AA=0;
                    double callrate=0; double ok_gen=0; double not_ok_gen=0;
                    double infoscore = 1;
                    int j = 0;
                    for (int i = 5; i < n; i+=3)
                    {
                        //determined
                        if(G.dosages==false)
                        {
                            if (S[j].isOK())
                            {
                                if (atof(tokens[i].c_str())>=.95){aa++;}
                                else if (atof(tokens[i+1].c_str())>=.95){aA++;}
                                else if (atof(tokens[i+2].c_str())>=.95){AA++;}
                            }
                            if (atof(tokens[i].c_str())>=.95){gen.push_back(2.0);ok_gen++;}
                            else if (atof(tokens[i+1].c_str())>=.95){gen.push_back(1.0);ok_gen++;}
                            else if (atof(tokens[i+2].c_str())>=.95){gen.push_back(0.0);ok_gen++;}
                            else {gen.push_back(-9999);not_ok_gen++;}
                        }
                        else
                        {
                            if (S[j].isOK())
                            {
                                aa+=atof(tokens[i].c_str());
                                aA+=atof(tokens[i+1].c_str());
                                AA+=atof(tokens[i+2].c_str());
                                double eij=(2*atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str());
                                double fij = (4*atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str());
                                fijeij+= fij - (eij*eij);
                            }
                            gen2.push_back(((atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str())));
                            gen.push_back(((atof(tokens[i].c_str())) + atof(tokens[i+1].c_str())));
                        }
                        if (n < i+5) i+=3;		//avoiding space in line end
                        
                        j++;
                    }
                    if (G.debugMode)cout<<"aa,aA,AA"<< aa<<","<<aA<<","<<AA<<"\n";
                    if (G.dosages)
                    {
                        callrate=1;
                        double eaf = 0;
                        if (aa+aA+AA>0)
                        {
                            eaf = ((2*aa)+aA)/(2*(aa+aA+AA));
                            
                        }
                        if (eaf==1 || eaf==0){infoscore=1;}
                        //info score calculation according to the measure in snptest:             
                        //https://mathgen.stats.ox.ac.uk/genetics_software/snptest/snptest.v2.pdf
                        else                                  
                        {   
                            infoscore = 1-(fijeij/(2*(aa+aA+AA)*eaf*(1-eaf)));
                        }
                        if (G.debugMode)cout<<"N: " << n << " eaf: " << eaf << " aa+aA+AA: " << aa+aA+AA << " Infoscore: "<< infoscore<<endl;
                        
                    }
                    else 
                    {
                        callrate = ok_gen/(ok_gen+not_ok_gen);   
                        if (G.debugMode)cout<<"Callrate: "<< callrate <<endl;
                    }
                    
                    std::stringstream mrk2;
                    mrk2 << chr << ":" << pos; 
                std:string mrkname = mrk2.str();
                    
                    bool markerExclusionInclusionIsOK=true;
                    
                    if (G.markerExclusion)
                    {
                        if (G.excludemarkers[mrkname]){markerExclusionInclusionIsOK=false;}
                    }
                    if (G.markerExtraction)
                    {
                        if (!G.extractmarkers[mrkname]){markerExclusionInclusionIsOK=false;}
                    }
                    
                    if (aa+aA+AA>0 && markerExclusionInclusionIsOK)
                    {
                        if (((2*aa)+aA)/(2*(aa+aA+AA))<=0.5)
                        {
                            marker m(markerName, effectAllele, nonEffectAllele, gen);
                            m.setMaf(((2*aa)+aA)/(2*(aa+aA+AA)));
                            if (pos>0 && chr>0 && chr<27 && G.rareVariantThreshold>=m.getMaf() && G.callrateThreshold<=callrate && G.infoThreshold<=infoscore && m.getMaf()>0)
                            {
                                m.setChromosome(chr);
                                m.setPosition(pos);
                                for (int zz=0; zz<GENE.size(); zz++)
                                {
                                    if (chr == GENE[zz].chr &&
                                        pos>= GENE[zz].start &&
                                        pos<=  GENE[zz].end)
                                    {
                                        GENE[zz].M.push_back(m);
                                    }
                                    else if (chr == GENE[zz].chr && pos>GENE[zz].end && GENE[zz].M.size()>0)
                                    {
                                        LowMemPrintGene(G, S, GENE[zz], LOG, ERR, OUTM, OUTF);
                                        GENE[zz].M.clear();
                                    }
                                }
                                m_ok++;
                                if (G.debugMode)cout<<"Marker added to array with maf: "<< ((2*aa)+aA)/(2*(aa+aA+AA))<<"\n";
                                if (G.debugMode)cout<<"Rare allele: "<< effectAllele <<"\n";
                            }
                            else
                            {
                                m_notok++;
                                if (G.debugMode)cout<<"Marker not added\n";
                            }
                        }
                        else
                        {
                            for (unsigned int j = 0; j < gen.size(); j++)
                            {
                                if (!G.dosages){if (gen[j]==2){gen[j]=0;} else if (gen[j]==0){gen[j]=2;}}
                                else {gen[j] = gen2[j];}
                                
                            } //turning around
                            marker m(markerName, nonEffectAllele, effectAllele, gen);
                            m.setMaf(((2*AA)+aA)/(2*(aa+aA+AA)));
                            if (pos>0 && chr>0 && chr<27 && G.rareVariantThreshold>=m.getMaf() && G.callrateThreshold<=callrate && G.infoThreshold<=infoscore && m.getMaf()>0)
                            {
                                m.setChromosome(chr);
                                m.setPosition(pos);
                                
                                for (int zz=0; zz<GENE.size(); zz++)
                                {
                                    if (chr == GENE[zz].chr &&
                                        pos>= GENE[zz].start &&
                                        pos<=  GENE[zz].end)
                                    {
                                        GENE[zz].M.push_back(m);
                                    }
                                    else if (chr == GENE[zz].chr && pos>GENE[zz].end && GENE[zz].M.size()>0)
                                    {
                                        LowMemPrintGene(G, S, GENE[zz], LOG, ERR, OUTM, OUTF);
                                        GENE[zz].M.clear();
                                    }
                                }
                                
                                m_ok++;
                                if (G.debugMode)cout<<"Marker added to array with maf: "<< ((2*AA)+aA)/(2*(aa+aA+AA))<<"\n";
                                if (G.debugMode)cout<<"Rare allele: "<< nonEffectAllele <<"\n";
                            }
                            else
                            {
                                m_notok++;
                                if (G.debugMode)cout<<"Marker not added\n";
                            }
                        }
                    }
                }
            }
            for (int zz=0; zz<GENE.size(); zz++)
            {
                if (GENE[zz].M.size()>0)
                {
                    LowMemPrintGene(G, S, GENE[zz], LOG, ERR, OUTM, OUTF);
                    GENE[zz].M.clear();
                }
            
            }

    }
    else
    {
        ifstream F (G.inputGenFile.c_str());
        if (F.is_open())
        {
            while (! F.eof() )
            {
                string line;
                vector<string> tokens;
                getline (F,line);
                string currentmarker = "";
                int n = Tokenize(string(line), tokens, " ");		//tabulating file by space
                if (n>2)
                {
                    int chr;
                    if (uc(tokens[0])=="MT") chr=26;
                    else if (uc(tokens[0])=="XY") chr=25;
                    else if (uc(tokens[0])=="Y") chr=24;
                    else if (uc(tokens[0])=="X") chr=23;
                    else chr = atoi(tokens[0].c_str());
                    
                    if (G.debugMode) cout << "Chromosome id: " << chr;
                    
                    if (forcedChromosome) 
                    {
                        chr = forcedChromosome;
                        if (G.debugMode) cout << "Chromosome forced: " << chr;                    
                    }
                    int pos = atoi(tokens[2].c_str());
                    string markerName = string(tokens[1]);
                    char effectAllele = tokens[3][0];
                    char nonEffectAllele = tokens[4][0];
                    if (G.debugMode) cout << "Pos: " << pos << "\nmarker:" << markerName << "\nea/nea:" << effectAllele <<"/" << nonEffectAllele<<"\n";
                    
                    vector <double> gen;
                    vector <double> gen2;
                    double fijeij = 0;                 //for info score
                    double aa=0; double aA=0; double AA=0;
                    double callrate=0; double ok_gen=0; double not_ok_gen=0;
                    double infoscore = 1;
                    int j = 0;
                    for (int i = 5; i < n; i+=3)
                    {
                        //determined
                        if(G.dosages==false)
                        {
                            if (S[j].isOK())
                            {
                                if (atof(tokens[i].c_str())>=.95){aa++;}
                                else if (atof(tokens[i+1].c_str())>=.95){aA++;}
                                else if (atof(tokens[i+2].c_str())>=.95){AA++;}
                            }
                            if (atof(tokens[i].c_str())>=.95){gen.push_back(2.0);ok_gen++;}
                            else if (atof(tokens[i+1].c_str())>=.95){gen.push_back(1.0);ok_gen++;}
                            else if (atof(tokens[i+2].c_str())>=.95){gen.push_back(0.0);ok_gen++;}
                            else {gen.push_back(-9999);not_ok_gen++;}
                        }
                        else
                        {
                            if (S[j].isOK())
                            {
                                aa+=atof(tokens[i].c_str());
                                aA+=atof(tokens[i+1].c_str());
                                AA+=atof(tokens[i+2].c_str());
                                double eij=(2*atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str());
                                double fij = (4*atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str());
                                fijeij+= fij - (eij*eij);
                            }
                            gen2.push_back(((atof(tokens[i+2].c_str())) + atof(tokens[i+1].c_str())));
                            gen.push_back(((atof(tokens[i].c_str())) + atof(tokens[i+1].c_str())));
                        }
                        if (n < i+5) i+=3;		//avoiding space in line end
                        
                        j++;
                    }
                    if (G.debugMode)cout<<"aa,aA,AA"<< aa<<","<<aA<<","<<AA<<"\n";
                    if (G.dosages)
                    {
                        callrate=1;
                        double eaf = 0;
                        if (aa+aA+AA>0)
                        {
                            eaf = ((2*aa)+aA)/(2*(aa+aA+AA));
                            
                        }
                        if (eaf==1 || eaf==0){infoscore=1;}
                        //info score calculation according to the measure in snptest:             
                        //https://mathgen.stats.ox.ac.uk/genetics_software/snptest/snptest.v2.pdf
                        else                                  
                        {   
                            infoscore = 1-(fijeij/(2*(aa+aA+AA)*eaf*(1-eaf)));
                        }
                        if (G.debugMode)cout<<"N: " << n << " eaf: " << eaf << " aa+aA+AA: " << aa+aA+AA << " Infoscore: "<< infoscore<<endl;
                        
                    }
                    else 
                    {
                        callrate = ok_gen/(ok_gen+not_ok_gen);   
                        if (G.debugMode)cout<<"Callrate: "<< callrate <<endl;
                    }
                    
                    std::stringstream mrk2;
                    mrk2 << chr << ":" << pos; 
                    string mrkname = mrk2.str();
                    
                    bool markerExclusionInclusionIsOK=true;
                    
                    if (G.markerExclusion)
                    {
                        if (G.excludemarkers[mrkname]){markerExclusionInclusionIsOK=false;}
                    }
                    if (G.markerExtraction)
                    {
                        if (!G.extractmarkers[mrkname]){markerExclusionInclusionIsOK=false;}
                    }
                    
                    if (aa+aA+AA>0 && markerExclusionInclusionIsOK)
                    {
                        if (((2*aa)+aA)/(2*(aa+aA+AA))<=0.5)
                        {
                            marker m(markerName, effectAllele, nonEffectAllele, gen);
                            m.setMaf(((2*aa)+aA)/(2*(aa+aA+AA)));
                            if (pos>0 && chr>0 && chr<27 && G.rareVariantThreshold>=m.getMaf() && G.callrateThreshold<=callrate && G.infoThreshold<=infoscore && m.getMaf()>0)
                            {
                                m.setChromosome(chr);
                                m.setPosition(pos);
                                for (int zz=0; zz<GENE.size(); zz++)
                                {
                                    if (chr == GENE[zz].chr &&
                                        pos>= GENE[zz].start &&
                                        pos<=  GENE[zz].end)
                                    {
                                        GENE[zz].M.push_back(m);
                                    }
                                    else if (chr == GENE[zz].chr && pos>GENE[zz].end && GENE[zz].M.size()>0)
                                    {
                                        LowMemPrintGene(G, S, GENE[zz], LOG, ERR, OUTM, OUTF);
                                        GENE[zz].M.clear();
                                    }
                                }
                                m_ok++;
                                if (G.debugMode)cout<<"Marker added to array with maf: "<< ((2*aa)+aA)/(2*(aa+aA+AA))<<"\n";
                                if (G.debugMode)cout<<"Rare allele: "<< effectAllele <<"\n";
                            }
                            else
                            {
                                m_notok++;
                                if (G.debugMode)cout<<"Marker not added\n";
                            }
                        }
                        else
                        {
                            for (unsigned int j = 0; j < gen.size(); j++)
                            {
                                if (!G.dosages){if (gen[j]==2){gen[j]=0;} else if (gen[j]==0){gen[j]=2;}}
                                else {gen[j] = gen2[j];}
                                
                            } //turning around
                            marker m(markerName, nonEffectAllele, effectAllele, gen);
                            m.setMaf(((2*AA)+aA)/(2*(aa+aA+AA)));
                            if (pos>0 && chr>0 && chr<27 && G.rareVariantThreshold>=m.getMaf() && G.callrateThreshold<=callrate && G.infoThreshold<=infoscore && m.getMaf()>0)
                            {
                                m.setChromosome(chr);
                                m.setPosition(pos);
                                
                                for (int zz=0; zz<GENE.size(); zz++)
                                {
                                    if (chr == GENE[zz].chr &&
                                        pos>= GENE[zz].start &&
                                        pos<=  GENE[zz].end)
                                    {
                                        GENE[zz].M.push_back(m);
                                    }
                                    else if (chr == GENE[zz].chr && pos>GENE[zz].end && GENE[zz].M.size()>0)
                                    {
                                        LowMemPrintGene(G, S, GENE[zz], LOG, ERR, OUTM, OUTF);
                                        GENE[zz].M.clear();
                                    }
                                }
                                
                                m_ok++;
                                if (G.debugMode)cout<<"Marker added to array with maf: "<< ((2*AA)+aA)/(2*(aa+aA+AA))<<"\n";
                                if (G.debugMode)cout<<"Rare allele: "<< nonEffectAllele <<"\n";
                            }
                            else
                            {
                                m_notok++;
                                if (G.debugMode)cout<<"Marker not added\n";
                            }
                        }
                    }
                }
            }
            for (int zz=0; zz<GENE.size(); zz++)
            {
                if (GENE[zz].M.size()>0)
                {
                    LowMemPrintGene(G, S, GENE[zz], LOG, ERR, OUTM, OUTF);
                    GENE[zz].M.clear();
                }
            }
            
        }
        else 
        {
            cerr << "Error: Cannot find or access " << G.inputGenFile << ". Exit program." << endl;
            sprintf(sb, "E%.8d" , G.errNr); G.errNr++;
            LOG << "Error: Cannot find or access " << G.inputGenFile << ". Exit program! (" << sb << ")" <<endl;
            ERR << sb << " Error: Cannot find or access " << G.inputGenFile << ". Program exit!" <<endl;
            ERR << sb << " Error: Make sure that this file is in given path and is readable." <<endl;
            return false;
        }
    }
	return true;
}

//
//

bool 
readMapData(global & G, vector <gene> & GENE,  ofstream & LOG, ofstream & ERR)
{
	char sb [10];				//error name char buffer
	ifstream F (G.inputMapFile.c_str());
	if (F.is_open())
    {
       	while (! F.eof() )
        {
	        string line;
        	vector<string> tokens;
        	getline (F,line);
			string currentmarker = "";
			int n = Tokenize(string(line), tokens, " ");		//tabulating file by space
            if (n>2)
            {
				gene g;
				g.name = string(tokens[0]);
				g.chr = atoi(tokens[1].c_str());
				g.start = atoi(tokens[2].c_str()) - G.flanking;
				g.end = atoi(tokens[3].c_str()) + G.flanking;
				
                if (G.chromosome == 0 || 
                    G.chromosome==g.chr)
                        GENE.push_back(g);
			}
		}
	}
	else 
	{
		cerr << "Error: Cannot find or access " << G.inputMapFile << ". Exit program." << endl;
		sprintf(sb, "E%.8d" , G.errNr); G.errNr++;
		LOG << "Error: Cannot find or access " << G.inputMapFile << ". Exit program! (" << sb << ")" <<endl;
		ERR << sb << " Error: Cannot find or access " << G.inputMapFile << ". Program exit!" <<endl;
		ERR << sb << " Error: Make sure that this file is in given path and is readable." <<endl;
		return false;
	}
	return true;
}


void printVersion(string version)
{
    cout << "GRANVIL version: " << version << endl;
    exit(0);
}
void printHelp(string version)
{
    string v;
    double len =17 -  version.size()-1;
    int f = (int) ((len/2.0)+0.6);
    int r = (int) ((len/2.0));
    for (int i=0; i < f;i++)v+=" ";
    v+='v';
    v+=version;
    for (int i=0; i < r;i++)v+=" ";
    cout << endl;
    cout << "@----------------------------------------------------------@" << endl;
    cout << "|     GRANVIL        |" << v << "|  September, 2012  |" << endl;
    cout << "|----------------------------------------------------------|" << endl;
    cout << "|           (C) 2008 Reedik Magi & Andrew P Morris         |" << endl;
    cout << "|               GNU General Public License, v2             |" << endl;
    cout << "|----------------------------------------------------------|" << endl;
    cout << "|   GRANVIL - Gene- or Region-based ANanlysis of Variants  |" << endl; 
    cout << "|             of Intermediate and Low frequency            |" << endl;
    cout << "|                                                          |" << endl;
    cout << "|  For documentation, citation & bug-report instructions:  |" << endl;
    cout << "|            http://www.well.ox.ac.uk/GRANVIL/             |" << endl;
    cout << "@----------------------------------------------------------@" << endl;
    cout << endl;	     
    cout << endl;	     
  
    exit (0);
}

void LowMemPrintGene(global & GLOBAL, vector <sample> & S, gene & GENE_p, ofstream & LOG, ofstream & ERR, ofstream & OUT)
{
    int n = GLOBAL.pheno_ok;	// number of samples for analysis
    int covs = (int)S[0].getCovariateCount()+2; //room for phenotype and constant
    arrayD *Y = new arrayD(n);
    arrayD *W = new arrayD(n);
    matrixD *X = new matrixD(n,covs);
    vector<marker> M;

	double totalMAF =0; 
    
    int count = 0;
    int count_all = 0;
    vector <std::string> marker_names;
    vector <double> genotype, weight;

    if (GLOBAL.debugMode)cout << "Working on gene: "<< GENE_p.name << endl;

        M = GENE_p.M;


    for (unsigned int k = 0; k < S.size(); k++)
    {
        genotype.push_back(0);
        weight.push_back(0);
    }

    for (unsigned int j = 0; j < M.size(); j++)
    {
        if (M[j].getChromosome() == GENE_p.chr
            && M[j].getPosition() >= GENE_p.start
            && M[j].getPosition() <= GENE_p.end)
        {
            if (GLOBAL.debugMode)cout << "Marker in region: "<< M[j].getName() << endl;
            if (GLOBAL.debugMode)cout << "MAF (thresh): "<< M[j].getMaf() <<"("<<GLOBAL.rareVariantThreshold<<")"<< endl;
            if (M[j].getMaf()<=GLOBAL.rareVariantThreshold && M[j].getMaf()>0)
            {
                if (GLOBAL.debugMode)cout << "MAF ok" << endl;
                for (unsigned int k = 0; k < S.size(); k++)
                {
                    if (!GLOBAL.dosages)
                    {
                        if (M[j].getGenotype(k)>0)
                        {
                            if(!GLOBAL.dosages)genotype[k]++;  //add one
                        }
                        else if(M[j].getGenotype(k)!=0)
                        {
                            weight[k]++; // we have missing phenotype
                        }
                    }
                    else
                    {
                        if (M[j].getGenotype(k)>=0)
                        {
                            genotype[k]+= M[j].getGenotype(k); //add dosage
                        }
                        else
                        {
                            weight[k]++;
                        }
                        
                    }
                }
                count++;
                totalMAF+=M[j].getMaf();
                marker_names.push_back(M[j].getName());
            }
            count_all++;
        }
    }
    if (count>0)			//this gene has one or more markers with MAF below the cut-off
    {
        if (GLOBAL.debugMode)cout << "Markers found: "<< count << endl;
        for (int l=0; l<marker_names.size(); l++){LOG << GENE_p.name << "\t" << marker_names[l]<<endl; }
        int m = 0;
        double rareCount =0;  // number of rare alleles counted from all individuals
        
        for (unsigned int k = 0; k < S.size(); k++)
        {
            if(S[k].isOK())
            {
                if (GLOBAL.quantitativeTrait)Y->put(m,S[k].getPhenotype());
                else Y->put(m,S[k].getStatus());
                X->put(m,0,1.0);
                if ((count-weight[k])>0){X->put(m, 1, (double) genotype[k]/ (double)(count-weight[k]) );}
                else {X->put(m, 1, 0 );}
                rareCount += genotype[k];
                for (int l = 0; l < S[0].getCovariateCount(); l++)
                {
                    X->put( m, l+2, S[k].getCovariate(l));
                }
                W->put( m, (double)(count- weight[k])/(count));
                //                        OUT << S[k].getStatus() << "\t" << X->get(m,1) << "\t" << W->get(m)<<endl;
            
                m++;
            }
        }
        OUT << GENE_p.name << "\t" << count <<"\t" << n << "\t" << rareCount << "\t" << totalMAF << "\t" << totalMAF/count << "\t";
        
        
        if (find(GLOBAL.printGene.begin(), GLOBAL.printGene.end(), GENE_p.name) != GLOBAL.printGene.end())
        {
            std::string genFileName = GENE_p.name + ".gene.model.txt";
            ofstream GENEPRINT (genFileName.c_str());
            printModel(X,Y,GENEPRINT);
            GENEPRINT.close();
            
        }
        
        if (GLOBAL.quantitativeTrait) //is  quantitative trait - linear regression
        {
            lr LR;
            if (LR.lr_w(Y,X,W))
            {
                double coef, se, z, p;
                coef = LR.Cstat->get(1);
                se = LR.SECstat->get(1);
                z = coef/se;
                if (se>0)
                {
                    p = (1-studenttdistribution(n-covs ,dabs(z)))*2;
                    if (p>1) p=1;
                    OUT << coef << "\t" << se << "\t" <<z<< "\t" <<p<< endl;
                }
                else
                {
                    ERR << "Stderr calculation problem for gene : " << GENE_p.name << endl;
                    OUT << "NA\tNA\tNA\tNA\tNA\tNA" << endl;
                }
            }
            else
            {
                ERR << "Singularity problem in analysis of gene: " << GENE_p.name << endl;
                OUT << "NA\tNA\tNA\tNA\tNA\tNA" << endl;
            }
            
        }
        else   //is not quantitative trait - logistic regression
        {
            lr LR;
            //X->print();
            if (LR.lg_w(Y,X,W))
            {
                double coef, se, z, p;
                coef = LR.Cstat->get(1);
                se = LR.SECstat->get(1);
                z = coef/se;
                if (se>0)
                {
                p = 1-chisquaredistribution(1,dabs(z)*dabs(z)); //chiprobP(Z*Z,1)
                OUT << coef << "\t" << se << "\t" <<z<< "\t" <<p<< endl;
                }
                else
                {
                   ERR << "Stderr calculation problem for gene : " << GENE_p.name << endl;
                   OUT << "NA\tNA\tNA\tNA\tNA\tNA" << endl; 
                }
            }
            else
            {
                ERR << "Singularity problem in analysis of gene: " << GENE_p.name << endl;
                OUT << "NA\tNA\tNA\tNA\tNA\tNA" << endl;
            }
        }
        genotype.clear();
        weight.clear();
        
    } // if count>0
}


void LowMemPrintGene(global & GLOBAL, vector <sample> & S, gene & GENE_p, ofstream & LOG, ofstream & ERR, ofstream & OUTM, ofstream & OUTF)
{
    int n_m = GLOBAL.pheno_men_ok;	// number of samples for men analysis
    int n_f = GLOBAL.pheno_women_ok;	// number of samples for women analysis
    int covs = (int)S[0].getCovariateCount()+2; //room for phenotype and constant
    
    arrayD *YM = new arrayD(n_m);
    arrayD *WM = new arrayD(n_m);
    matrixD *XM = new matrixD(n_m,covs);
    
    arrayD *YF = new arrayD(n_f);
    arrayD *WF = new arrayD(n_f);
    matrixD *XF = new matrixD(n_f,covs);

    vector<marker> M;
    
	double totalMAF =0; 
    
    int count = 0;
    int count_all = 0;
    vector <std::string> marker_names;
    vector <double> genotype, weight;
    
    if (GLOBAL.debugMode)cout << "Working on gene: "<< GENE_p.name << endl;
    
        M = GENE_p.M;

    
    for (unsigned int k = 0; k < S.size(); k++)
    {
        genotype.push_back(0);
        weight.push_back(0);
    }
    
    for (unsigned int j = 0; j < M.size(); j++)
    {
        if (M[j].getChromosome() == GENE_p.chr
            && M[j].getPosition() >= GENE_p.start
            && M[j].getPosition() <= GENE_p.end)
        {
            if (GLOBAL.debugMode)cout << "Marker in region: "<< M[j].getName() << endl;
            if (GLOBAL.debugMode)cout << "MAF (thresh): "<< M[j].getMaf() <<"("<<GLOBAL.rareVariantThreshold<<")"<< endl;
            if (M[j].getMaf()<=GLOBAL.rareVariantThreshold && M[j].getMaf()>0)
            {
                if (GLOBAL.debugMode)cout << "MAF ok" << endl;
                for (unsigned int k = 0; k < S.size(); k++)
                {
                    if (!GLOBAL.dosages)
                    {
                        if (M[j].getGenotype(k)>0)
                        {
                            if(!GLOBAL.dosages)genotype[k]++;  //add one
                        }
                        else if(M[j].getGenotype(k)!=0)
                        {
                            weight[k]++; // we have missing phenotype
                        }
                    }
                    else
                    {
                        if (M[j].getGenotype(k)>=0)
                        {
                            genotype[k]+= M[j].getGenotype(k); //add dosage
                        }
                        else
                        {
                            weight[k]++;
                        }
                        
                    }
                }
                count++;
                totalMAF+=M[j].getMaf();
                marker_names.push_back(M[j].getName());
            }
            count_all++;
        }
    }
    if (count>0)			//this gene has one or more markers with MAF below the cut-off
    {
        if (GLOBAL.debugMode)cout << "Markers found: "<< count << endl;
        for (int l=0; l<marker_names.size(); l++){LOG << GENE_p.name << "\t" << marker_names[l]<<endl; }
        
 //MEN ANALYSIS       
        if (n_m>0)
        {
            int m = 0;
            double rareCount =0;  // number of rare alleles counted from all individuals
            for (unsigned int k = 0; k < S.size(); k++)
            {
                if(S[k].isOK() && S[k].getSex()==0)
                {
                    if (GLOBAL.quantitativeTrait)YM->put(m,S[k].getPhenotype());
                    else YM->put(m,S[k].getStatus());
                    XM->put(m,0,1.0);
                    if ((count-weight[k])>0){XM->put(m, 1, (double) genotype[k]/ (double)(count-weight[k]) );}
                    else {XM->put(m, 1, 0 );}
                    rareCount += genotype[k];
                    for (int l = 0; l < S[0].getCovariateCount(); l++)
                    {
                        XM->put( m, l+2, S[k].getCovariate(l));
                    }
                    WM->put( m, (double)(count- weight[k])/(count));
                    //                        OUT << S[k].getStatus() << "\t" << X->get(m,1) << "\t" << W->get(m)<<endl;
                    
                    m++;
                }
            }
            OUTM << GENE_p.name << "\t" << count <<"\t" << n_m << "\t" << rareCount << "\t" << totalMAF << "\t" << totalMAF/count << "\t";
            
            
            if (find(GLOBAL.printGene.begin(), GLOBAL.printGene.end(), GENE_p.name) != GLOBAL.printGene.end())
            {
                std::string genFileName = GENE_p.name + ".men.gene.model.txt";
                ofstream GENEPRINT (genFileName.c_str());
                printModel(XM,YM,GENEPRINT);
                GENEPRINT.close();
                
            }
            
            if (GLOBAL.quantitativeTrait) //is  quantitative trait - linear regression
            {
                lr LR;
                if (LR.lr_w(YM,XM,WM))
                {
                    double coef, se, z, p;
                    coef = LR.Cstat->get(1);
                    se = LR.SECstat->get(1);
                    z = coef/se;
                    if (se>0 && n_m-covs>0)
                    {
                        p = (1-studenttdistribution(n_m-covs ,dabs(z)))*2;
                        if (p>1) p=1;
                        OUTM << coef << "\t" << se << "\t" <<z<< "\t" <<p<< endl;
                    }
                    else
                    {
                        ERR << "Stderr calculation problem or too small samplesize in men analysis for gene : " << GENE_p.name << endl;
                        OUTM << "NA\tNA\tNA\tNA\tNA\tNA" << endl;
                    }
                }
                else
                {
                    ERR << "Singularity problem in men analysis of gene: " << GENE_p.name << endl;
                    OUTM << "NA\tNA\tNA\tNA\tNA\tNA" << endl;
                }
                
            }
            else   //is not quantitative trait - logistic regression
            {
                lr LR;
                //X->print();
                if (LR.lg_w(YM,XM,WM))
                {
                    double coef, se, z, p;
                    coef = LR.Cstat->get(1);
                    se = LR.SECstat->get(1);
                    z = coef/se;
                    if (se>0)
                    {
                        p = 1-chisquaredistribution(1,dabs(z)*dabs(z)); //chiprobP(Z*Z,1)
                        OUTM << coef << "\t" << se << "\t" <<z<< "\t" <<p<< endl;
                    }
                    else
                    {
                        ERR << "Stderr calculation problem in men analysis for gene : " << GENE_p.name << endl;
                        OUTM << "NA\tNA\tNA\tNA\tNA\tNA" << endl; 
                    }
                }
                else
                {
                    ERR << "Singularity problem in men analysis of gene: " << GENE_p.name << endl;
                    OUTM << "NA\tNA\tNA\tNA\tNA\tNA" << endl;
                }
            }
        }
        
/////WOMEN   
        if (n_f>0)
        {
            int m = 0;
            double rareCount =0;  // number of rare alleles counted from all individuals
            for (unsigned int k = 0; k < S.size(); k++)
            {
                if(S[k].isOK() && S[k].getSex()==1)
                {
                    if (GLOBAL.quantitativeTrait)YF->put(m,S[k].getPhenotype());
                    else YF->put(m,S[k].getStatus());
                    XF->put(m,0,1.0);
                    if ((count-weight[k])>0){XF->put(m, 1, (double) genotype[k]/ (double)(count-weight[k]) );}
                    else {XF->put(m, 1, 0 );}
                    rareCount += genotype[k];
                    for (int l = 0; l < S[0].getCovariateCount(); l++)
                    {
                        XF->put( m, l+2, S[k].getCovariate(l));
                    }
                    WF->put( m, (double)(count- weight[k])/(count));
                    //                        OUT << S[k].getStatus() << "\t" << X->get(m,1) << "\t" << W->get(m)<<endl;
                    
                    m++;
                }
            }
            OUTF << GENE_p.name << "\t" << count <<"\t" << n_f << "\t" << rareCount << "\t" << totalMAF << "\t" << totalMAF/count << "\t";
            
            
            if (find(GLOBAL.printGene.begin(), GLOBAL.printGene.end(), GENE_p.name) != GLOBAL.printGene.end())
            {
                std::string genFileName = GENE_p.name + ".women.gene.model.txt";
                ofstream GENEPRINT (genFileName.c_str());
                printModel(XF,YF,GENEPRINT);
                GENEPRINT.close();
                
            }
            
            if (GLOBAL.quantitativeTrait) //is  quantitative trait - linear regression
            {
                lr LR;
                if (LR.lr_w(YF,XF,WF))
                {
                    double coef, se, z, p;
                    coef = LR.Cstat->get(1);
                    se = LR.SECstat->get(1);
                    z = coef/se;
                    if (se>0 && n_m-covs>0)
                    {
                        p = (1-studenttdistribution(n_f-covs ,dabs(z)))*2;
                        if (p>1) p=1;
                        OUTF << coef << "\t" << se << "\t" <<z<< "\t" <<p<< endl;
                    }
                    else
                    {
                        ERR << "Stderr calculation or too small samplesize problem in women analysis for gene : " << GENE_p.name << endl;
                        OUTF << "NA\tNA\tNA\tNA\tNA\tNA" << endl;
                    }
                }
                else
                {
                    ERR << "Singularity problem in women analysis of gene: " << GENE_p.name << endl;
                    OUTF << "NA\tNA\tNA\tNA\tNA\tNA" << endl;
                }
                
            }
            else   //is not quantitative trait - logistic regression
            {
                lr LR;
                //X->print();
                if (LR.lg_w(YF,XF,WF))
                {
                    double coef, se, z, p;
                    coef = LR.Cstat->get(1);
                    se = LR.SECstat->get(1);
                    z = coef/se;
                    if (se>0)
                    {
                        p = 1-chisquaredistribution(1,dabs(z)*dabs(z)); //chiprobP(Z*Z,1)
                        OUTF << coef << "\t" << se << "\t" <<z<< "\t" <<p<< endl;
                    }
                    else
                    {
                        ERR << "Stderr calculation problem in women analysis for gene : " << GENE_p.name << endl;
                        OUTF << "NA\tNA\tNA\tNA\tNA\tNA" << endl; 
                    }
                }
                else
                {
                    ERR << "Singularity problem in women analysis of gene: " << GENE_p.name << endl;
                    OUTF << "NA\tNA\tNA\tNA\tNA\tNA" << endl;
                }
            }
        }
        
        
        
        genotype.clear();
        weight.clear();
        
    } // if count>0
}


bool 
readExclusionMarkersData(global & G, vector <sample> & S,  ofstream & LOG, ofstream & ERR)
{
	char sb [10];				//error name char buffer
	ifstream F (G.inputExclusionMarkersFile.c_str());
	if (F.is_open())
    {
       	while (! F.eof() )
        {
	        string line;
        	vector<string> tokens;
        	getline (F,line);
			string currentmarker = "";
			int n = Tokenize(string(line), tokens, " ");		//tabulating file by space
			if (n>0)
			{
                G.excludemarkers[string(tokens[0]) + ":" + string(tokens[3])]= true;
			}	
		}
        
	}
	else
	{
		cerr << "Error: Cannot find or access " << G.inputExclusionMarkersFile << ". Exit program." << endl;
		sprintf(sb, "E%.8d" , G.errNr); G.errNr++;
		LOG << "Error: Cannot find or access " << G.inputExclusionMarkersFile << ". Exit program! (" << sb << ")" <<endl;
		ERR << sb << " Error: Cannot find or access " << G.inputExclusionMarkersFile << ". Program exit!" <<endl;
		ERR << sb << " Error: Make sure that this file is in given path and is readable." <<endl;
		return false;
	}

	return true;
}

bool 
readExtractMarkersData(global & G, vector <sample> & S,  ofstream & LOG, ofstream & ERR)
{
	char sb [10];				//error name char buffer
	ifstream F (G.inputExtractMarkersFile.c_str());
	if (F.is_open())
    {
       	while (! F.eof() )
        {
	        string line;
        	vector<string> tokens;
        	getline (F,line);
			string currentmarker = "";
			int n = Tokenize(string(line), tokens, " ");		//tabulating file by space
			if (n>0)
			{
				G.extractmarkers[string(tokens[0]) + ":" + string(tokens[3])]= true;
			}	
		}
        
	}
	else
	{
		cerr << "Error: Cannot find or access " << G.inputExtractMarkersFile << ". Exit program." << endl;
		sprintf(sb, "E%.8d" , G.errNr); G.errNr++;
		LOG << "Error: Cannot find or access " << G.inputExtractMarkersFile << ". Exit program! (" << sb << ")" <<endl;
		ERR << sb << " Error: Cannot find or access " << G.inputExtractMarkersFile << ". Program exit!" <<endl;
		ERR << sb << " Error: Make sure that this file is in given path and is readable." <<endl;
		return false;
	}
	return true;
}

void printModel( matrixD * X , arrayD * Y,ofstream & GENEPRINT)
{
    for (int i = 0 ; i < X->getRows(); i++)
    {
        GENEPRINT << Y->get(i);
            for (int j = 1; j < X->getCols(); j++)
            {
                GENEPRINT << "\t" << X->get(i,j) ;                
            }
        GENEPRINT << endl;
    }
}
